--[[

Name: Epeen
Revision: $Revision: 27401 $
Date: $Date: 2007-02-06 23:54:05 -0500 (Tue, 06 Feb 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: PvP Statistics and KoS System
Dependencies: Ace2

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 27401 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("Epeen")

L:RegisterTranslations("enUS", function() return {
    ["Epeen"] = true,
	["Options"] = true,
	["Shows the Options Dropdown"] = true,
	["Settings"] = true,
	["Change Settings"] = true,
	["Change Epeen Settings"] = true,
	["General Settings"] = true,
	["Advanced Settings"] = true,
	["Reset DB to Defaults"] = true,
	["Reset the Database back to Defaults"] = true,
	["Options you may not want to mess with"] = true,
	
} end)

--- ========================================================= ---
--- Add ParserLib
--- ========================================================= ---

local parser = ParserLib:GetInstance("1.1")

--- ========================================================= ---
--- Register DB for Music Stream
--- ========================================================= ---
Epeen = AceLibrary("AceAddon-2.0"):new("AceComm-2.0", "AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "AceEvent-2.0", "AceHook-2.1", "AceModuleCore-2.0")
Epeen.dewdrop = AceLibrary("Dewdrop-2.0")

Epeen:RegisterDB("EpeenDB")
Epeen:SetModuleMixins("AceComm-2.0", "AceConsole-2.0", "AceEvent-2.0", "AceHook-2.1", "AceDebug-2.0")

Epeen.defaultDB = {
}

local bluecolor = "|cFF0099CC"
local lbluecolor = "|cFF77BBFF"
local redcolor = "|cFFFF5522"
local greycolor = "|c77AAAAAA"

Epeen.Options = {type = "group", handler = Epeen, args = {
		header = {
			type = "header",
			name = bluecolor..L["Epeen"].."  |c88888888"..Revision,
			icon = "Interface\\Addons\\Epeen\\icon.tga",
			iconHeight = 28,
			iconWidth = 28,
			order = 1
		}, 
		
		settingsheaderblank = {
			type = 'header',
			order = 299,
		},
		settingsheader = {
			type = "header",
			name = lbluecolor..L["Settings"],
			icon = "Interface\\GossipFrame\\BinderGossipIcon",
			order = 300
		}, 
		settings = {
			name = L["Change Settings"],
			desc = L["Change Epeen Settings"],
			order = 301,
			type = 'group',
			args = {
				generalheader = {
					type = "header",
					name = lbluecolor..L["General Settings"],
					icon = "Interface\\GossipFrame\\BinderGossipIcon",
					order = 100
				}, 
				advancedheaderblank = {
					type = 'header',
					order = 999,
				},
				advancedheader = {
					type = "header",
					name = lbluecolor..L["Advanced Settings"],
					icon = "Interface\\GossipFrame\\BinderGossipIcon",
					order = 1000
				},
				advancedsettings = {
					name = greycolor..L["Advanced Settings"],
					desc = greycolor..L["Options you may not want to mess with"],
					order = 1001,
					type = 'group',
					args = {
						header = {
							type = "header",
							name = lbluecolor..L["Advanced Settings"],
							icon = "Interface\\GossipFrame\\BinderGossipIcon",
							order = 1
						},
						blank = {
							type = 'header',
							order = 2,
						},
						reset = {
							name = redcolor..L["Reset DB to Defaults"],
							desc = greycolor..L["Reset the Database back to Defaults"],
							order = 200,
							type = "execute",
							confirm = true,
							func = function() Epeen:ResetDB(); end,
						},
					},
				},
			},
		},
		
		options = {
			name = L["Options"],
			desc = L["Shows the Options Dropdown"],
			order = 300,
			type = "execute",
			func = function() Epeen:OpenMenu(UIParent) end,
			guiHidden = true
		},
    }
}

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function Epeen:OnInitialize()
	Epeen:RegisterDefaults("profile", {
		debug = false,
	})
	
	self:RegisterEvent("Ace2_AddonDisabled")
	self:RegisterEvent("Ace2_AddonEnabled")
	self:RegisterEvent("AceEvent_FullyInitialized")
end

--- ========================================================= ---
--- On Full Initialize
--- ========================================================= ---

function Epeen:AceEvent_FullyInitialized()
	self:SetDebugging(self.db.profile.debug)
end

--- ========================================================= ---
--- On Addon Disabled
--- ========================================================= ---

function Epeen:Ace2_AddonDisabled(mod)
	self:Debug("Ace2_AddonDisabled - "..mod.name)
end

--- ========================================================= ---
--- On Addon Enabled
--- ========================================================= ---

function Epeen:Ace2_AddonEnabled(mod)
	self:Debug("Ace2_AddonEnabled - "..mod.name)
end

--- ========================================================= ---
--- On Debug Enable
--- ========================================================= ---

function Epeen:OnDebugEnable()
	self.db.profile.debug = true
	self:Debug("Debug - ON")
end	

--- ========================================================= ---
--- On Debug Enable
--- ========================================================= ---

function Epeen:OnDebugDisable()
	self.db.profile.debug = false
	self:Debug("Debug - OFF")
end

--- ========================================================= ---
--- Register Chat Command
--- ========================================================= ---

Epeen:RegisterChatCommand({"/epeen"}, Epeen.Options)

--- ========================================================= ---
--- Options Dropdown
--- ========================================================= ---

function Epeen:OpenMenu(frame)
	self.dewdrop:Open(frame,
		'children', function(level, value)
				self.dewdrop:FeedAceOptionsTable(self.Options)
			end,
		'cursorX', true,
		'cursorY', true
	)
end

--- ========================================================= ---
--- Get Fu Options
--- ========================================================= ---

function Epeen:GetFuOptions()
	Epeen.Options.args.settings.args = {
		name = L["FuBar Options"],
        desc = L["A module to change FuBar Options."],
        type = "group",
        args = {
		},
	}
end

--- ========================================================= ---
--- ResetDB
--- ========================================================= ---

function Epeen:ResetDB()
	for name, module in self:IterateModules() do
		if type(module.ResetDB) == "function" then
			module:ResetDB();
			self:Debug(name.." successfully reset.")
		end
	end
end