﻿--[[

Name: Epeen Alert
Revision: $Revision: 27401 $
Date: $Date: 2007-02-06 23:54:05 -0500 (Tue, 06 Feb 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: A module to alert you when when events are fired.
Dependencies: Ace2, Epeen

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 27401 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("Alert")

L:RegisterTranslations("enUS", function() return {
    ["Alert"] = true,
    ["Toggle alerting you when events are fired"] = true,
	
    ["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
	
	["Show"] = true,
    ["Toggle showing where to display the messages"] = true,
	
	["Show UIMessageFrame"] = true,
    ["Toggles showing message in UI Message Frame"] = true,
	["Show ChatFrame"] = true,
    ["Toggles showing message in Chat Frame"] = true,
	
	["Tooltip Settings"] = true,
    ["Change the Alert Tooltip Settings"] = true,
	["Alert Cooldown"] = true,
    ["Set the Alert Cooldown in seconds"] = true,

	["Play Sound"] = true,
	["Toggles playing sound"] = true,
} end)

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---

EpeenAlert = Epeen:NewModule("Alert")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function EpeenAlert:OnInitialize()
    self.db = Epeen:AcquireDBNamespace("Alert")
	
    EpeenAlert.defaultDB = {
        on = true,
		show = {true, false},
		cooldown = 3,
		sound = true,
    }
	
	Epeen:RegisterDefaults("Alert", "profile", opts or EpeenAlert.defaultDB or {})
	
    Epeen:ToggleModuleActive("Alert", self.db.profile.on)
  
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
    Epeen.Options.args.settings.args.Alert = {
        name = L["Alert"],
        desc = L["Toggle alerting you when events are fired"],
        type = "group",
		order = 101,
        args = { },
	}
	
	Epeen.Options.args.settings.args.Alert.args.AlertHeader = {
		name = bluecolor..L["Alert"].."  |c88888888"..Revision,
		icon = "Interface\\Addons\\Epeen\\icon.tga",
		iconHeight = 14,
		iconWidth = 14,
		order = 1,
		type = "header",
    }
	
	Epeen.Options.args.settings.args.Alert.args.AlertHeaderBlank = {
		order = 2,
		type = "header",
	}
	
	Epeen.Options.args.settings.args.Alert.args.AlertShow = {
        name = L["Show"],
        desc = L["Toggle showing where to display the messages"],
		order = 100,
        type = "group",
        args = {
        	chatframe = {
            	name = string.format(L["Show ChatFrame"], 2),
                desc = string.format(L["Toggles showing message in Chat Frame"], 2),
				order = 1,
                type = "toggle",
                get = function() return self.db.profile.show[2] end,
                set = function(v) self.db.profile.show[2] = v end
            },
			uimessageframe = {
                name = string.format(L["Show UIMessageFrame"], 1),
                desc = string.format(L["Toggles showing message in UI Message Frame"], 1),
				order = 2,
                type = "toggle",
                get = function() return self.db.profile.show[1] end,
                set = function(v) self.db.profile.show[1] = v end
            },
		},
	}
	
	Epeen.Options.args.settings.args.Alert.args.AlertToggleBlank = {
       	type = "header",
		order = 195,
	}
	
    Epeen.Options.args.settings.args.Alert.args.AlertToggle = {
    	name = L["Toggle"],
        desc = L["Toggle the Module On and Off"],
		order = 200,
        type = "toggle",
        get = function() return self.db.profile.on end,
        set = function() self.db.profile.on = Epeen:ToggleModuleActive("Alert") end
    }
	
end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function EpeenAlert:OnEnable()
	self:RegisterEvent("EpeenAlert");
end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function EpeenAlert:ResetDB()
	copyTable(self.db.profile, EpeenAlert.defaultDB)
end

--- ========================================================= ---
--- Alert
--- ========================================================= ---

function EpeenAlert:EpeenAlert(AlertWhere, AlertType, AlertName, AlertReason, AlertSound)
	if (self.db.profile.on == true) then
	
		if AlertReason == "" then
			AlertReason = nil;
		end
		
		if AlertWhere == "Combat" then
			AlertName = AlertName .. " (In Area)";
		end
		
		if (self.db.profile.show[1] == true) then
			if (IsAddOnLoaded("SCT")) then
				if not AlertReason then
					SCT:DisplayCustomEvent(AlertType.."  - "..AlertName,{r=1,g=1,b=1}, iscrit);
				else
					SCT:DisplayCustomEvent(AlertType.."  - "..AlertName.." : "..AlertReason,{r=1,g=1,b=1}, iscrit);
				end
			else
				if not AlertReason then
					UIErrorsFrame:AddMessage(AlertType.."  - "..AlertName,1, 1, 1, 1,10);
				else
					UIErrorsFrame:AddMessage(AlertType.."  - "..AlertName.." : "..AlertReason,1, 1, 1, 1,10);
				end
			end
		end
		if (self.db.profile.show[2] == true) then
			if not AlertReason then
				DEFAULT_CHAT_FRAME:AddMessage(AlertType.."  - "..AlertName);
			else
				DEFAULT_CHAT_FRAME:AddMessage(AlertType.."  - "..AlertName.." : "..AlertReason);
			end
		end
		
		if (AlertSound) then
			PlaySoundFile("Sound\\interface\\AuctionWindowOpen.wav");
		end		
	end
end