﻿--[[

Name: Epeen Combat
Revision: $Revision: 27401 $
Date: $Date: 2007-02-06 23:54:05 -0500 (Tue, 06 Feb 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: A module to scan combat messages.
Dependencies: Ace2, Epeen

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 27401 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("Combat")

L:RegisterTranslations("enUS", function() return {
	["Combat"] = true,
    ["Scan Combat Messages for Players/Enemies"] = true,
	["Alert"] = true,
	["Toggle whether or not to alert on Combat"] = true,		
	["Play Alert Sound"] = true,
	["Toggle whether or not to play sound on alert"] = true,
	["Alert Cooldown"] = true,
	["Set the Alert Cooldown in minutes"] = true,
	["Track"] = true,
    ["Toggle whether or not to track on Combat"] = true,
    ["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
} end)

--- ========================================================= ---
--- Add ParserLib
--- ========================================================= ---

local parser = ParserLib:GetInstance("1.1")

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---

EpeenCombat = Epeen:NewModule("Combat")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function EpeenCombat:OnInitialize()
    self.db = Epeen:AcquireDBNamespace("Combat")
	
    EpeenCombat.defaultDB = {
        on = true,
		alert = true,
		sound = true,
		cooldown = 1,
    }
	
	Epeen:RegisterDefaults("Combat", "profile", opts or EpeenCombat.defaultDB or {})
	
    Epeen:ToggleModuleActive("Combat", self.db.profile.on)
  
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
    Epeen.Options.args.settings.args.Combat = {
        name = L["Combat"],
        desc = L["Scan Combat Messages for Players/Enemies"],
        type = "group",
		order = 102,
        args = { },
	}
	
	Epeen.Options.args.settings.args.Combat.args.CombatHeader = {
		name = bluecolor..L["Combat"].."  |c88888888"..Revision,
		icon = "Interface\\Addons\\Epeen\\icon.tga",
		iconHeight = 14,
		iconWidth = 14,
		order = 1,
		type = "header",
    }
	
	Epeen.Options.args.settings.args.Combat.args.CombatHeaderBlank = {
		order = 2,
		type = "header",
	}

	Epeen.Options.args.settings.args.Combat.args.CombatAlert = {
        name = L["Alert"],
        desc = L["Toggle whether or not to alert on Combat"],
		order = 100,
        type = "toggle",
        get = function() return self.db.profile.alert end,
        set = function() self.db.profile.alert = not(self.db.profile.alert) end
    }
	
	Epeen.Options.args.settings.args.Combat.args.CombatSound = {
        name = L["Play Alert Sound"],
        desc = L["Toggle whether or not to play sound on alert"],
		order = 101,
        type = "toggle",
        get = function() return self.db.profile.sound end,
        set = function() self.db.profile.sound = not(self.db.profile.sound) end
    }
	
	Epeen.Options.args.settings.args.Combat.args.CombatCooldown = {
		name = L["Alert Cooldown"],
		desc = L["Set the Alert Cooldown in minutes"],
		order = 102,
		type = "range",
		min  = 1.0,
		max  = 10.0,
		step = 1,
		set  = function(cooldown) self.db.profile.cooldown = cooldown; end,
		get  = function() return self.db.profile.cooldown end,
    }
				
	Epeen.Options.args.settings.args.Combat.args.CombatToggleBlank = {
       	type = "header",
		order = 195,
	}
	
    Epeen.Options.args.settings.args.Combat.args.CombatToggle = {
    	name = L["Toggle"],
        desc = L["Toggle the Module On and Off"],
		order = 200,
        type = "toggle",
        get = function() return self.db.profile.on end,
        set = function() self.db.profile.on = Epeen:ToggleModuleActive("Alert") end
    }
	
	local playerName = nil;
	local playerFaction = nil;
	
end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function EpeenCombat:OnEnable()
	playerName = UnitName("player");
	playerFaction = UnitFactionGroup("player");
	
	-- enemy
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", function(event, info) self:EventHandler(event, info, "enemy") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES", function(event, info) self:EventHandler(event, info, "enemy") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF", function(event, info) self:EventHandler(event, info, "enemy") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", function(event, info) self:EventHandler(event, info, "enemy") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", function(event, info) self:EventHandler(event, info, "enemy") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_COMBAT_HOSTILE_DEATH", function(event, info) self:EventHandler(event, info, "enemy") end)

	-- friendly
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS", function(event, info) self:EventHandler(event, info, "friendly") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES", function(event, info) self:EventHandler(event, info, "friendly") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF", function(event, info) self:EventHandler(event, info, "friendly") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE", function(event, info) self:EventHandler(event, info, "friendly") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE", function(event, info) self:EventHandler(event, info, "friendly") end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_COMBAT_FRIENDLY_DEATH", function(event, info) self:EventHandler(event, info, "friendly") end)

	-- buffs
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF", function(event, info) self:EventHandler(event, info) end) -- curse
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenCombat", "CHAT_MSG_SPELL_AURA_GONE_OTHER", function(event, info) self:EventHandler(event, info) end)

end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function EpeenCombat:ResetDB()
	copyTable(self.db.profile, EpeenCombat.defaultDB)
end

--- ========================================================= ---
--- Combat Messages
--- ========================================================= ---

function EpeenCombat:EventHandler(event, info)
	if (self.db.profile.on == true) then
	
		if EpeenCombatCooldown then
			return;
		end
		
		if info.source then
			sourcevictim = info.source;
		elseif info.victim then
			sourcevictim = info.victim;
		end
				
		if (EpeenKoS.db.profile.kosplayer[string.lower(sourcevictim)]) then
			AlertType = "Player";
			AlertName = sourcevictim;
			AlertReason = EpeenKoS.db.profile.kosplayer[string.lower(sourcevictim)].reason;
		
			if AlertReason == "" then
				AlertReason = nil;
			end
		
			EpeenCombatCooldown = true;
		
			self:ScheduleEvent("Cooldown", self.Cooldown, (self.db.profile.cooldown * 60), self)
			self:TriggerEvent("EpeenAlert", "Combat", AlertType.." KoS", AlertName, AlertReason, self.db.profile.sound)
		elseif EpeenLastSeen.db.profile.chars[sourcevictim] then
			if EpeenLastSeen.db.profile.chars[sourcevictim].guild then
				local guildName = EpeenLastSeen.db.profile.chars[sourcevictim].guild;
				testpos = EpeenLastSeen.db.profile.chars[sourcevictim].guild;
				local guildnamelc = string.gsub(string.lower(guildName), "%s", "_");
				
				if (EpeenKoS.db.profile.kosguild[guildnamelc]) then
					AlertType = "Guild";
					AlertName = sourcevictim;
					AlertReason = EpeenKoS.db.profile.kosguild[guildnamelc].reason;
		
					if AlertReason == "" then
						AlertReason = nil;
					end
			
					EpeenCombatCooldown = true;
		
					self:ScheduleEvent("Cooldown", self.Cooldown, (self.db.profile.cooldown * 60), self)
					self:TriggerEvent("EpeenAlert", "Combat", AlertType.." KoS", AlertName, AlertReason, self.db.profile.sound)
				end
			end
		end
	end
end

--- ========================================================= ---
--- Cooldown
--- ========================================================= ---

function EpeenCombat:Cooldown()
	EpeenCombatCooldown = nil;
end