﻿--[[

Name: Epeen Friendly
Revision: $Revision: 23844 $
Date: $Date: 2007-01-07 03:01:22 -0500 (Sun, 07 Jan 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: A module to keep track of Friendly System.
Dependencies: Ace2, Epeen

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 23844 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("Friendly")

L:RegisterTranslations("enUS", function() return {
    ["Friendly"] = true,
    ["Toggle keeping track of Friendly Players"] = true,
	
	["Add Player"] = true,
	["Add Player to Friendly Database"] = true,
	["Add Guild"] = true,
	["Add Guild to Friendly Database"] = true,
	
    ["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
	
	["Friendly"] = true,
	
	["Name"] = true,
	["Reason"] = true,
	
	["Friendly Player"] = true,
	["Friendly Guild"] = true,
	
	["Reset"] = true,
	["Remove"] = true,
	["Accept"] = true,
	["Cancel"] = true,
} end)

EPEEN_KOS_LABEL = L["Friendly"]

EPEEN_KOS_NAME = L["Name"]
EPEEN_KOS_REASON = L["Reason"]

EPEEN_RESET = L["Reset"]
EPEEN_REMOVE = L["Remove"]
EPEEN_ACCEPT = L["Accept"]
EPEEN_CANCEL = L["Cancel"]

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---
EpeenFriendly = Epeen:NewModule("Friendly")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function EpeenFriendly:OnInitialize()
    self.db = Epeen:AcquireDBNamespace("Friendly")
	
    EpeenFriendly.defaultDB = {
		friendlyplayer = { },
		friendlyguild = { },
        on = true,
    }
	
	Epeen:RegisterDefaults("Friendly", "profile", opts or EpeenFriendly.defaultDB or {})
	
    Epeen:ToggleModuleActive("Friendly", self.db.profile.on)
	
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
	Epeen.Options.args.settings.args.Friendly = {
        name = L["Friendly"],
        desc = L["Toggle keeping track of Friendly Players"],
        type = "group",
		order = 103,
        args = { },
	}
	
	Epeen.Options.args.settings.args.Friendly.args.FriendlyHeader = {
		name = bluecolor..L["Friendly"].."  |c88888888"..Revision,
		icon = "Interface\\Addons\\Epeen\\icon.tga",
		iconHeight = 14,
		iconWidth = 14,
		order = 1,
		type = "header",
    }
	
	Epeen.Options.args.settings.args.Friendly.args.FriendlyHeaderBlank = {
		order = 2,
		type = "header",
	}
	
	Epeen.Options.args.settings.args.Friendly.args.FriendlyPlayer = {
        name = L["Add Player"],
        desc = L["Add Player to Friendly Database"],
		order = 100,
        type = "execute",
    	func = function() self.PlayerPopup() end,
	}
	
	Epeen.Options.args.settings.args.Friendly.args.FriendlyGuild = {
        name = L["Add Guild"],
        desc = L["Add Guild to Friendly Database"],
		order = 101,
        type = "execute",
    	func = function() self.GuildPopup() end,
	}
	
	Epeen.Options.args.settings.args.Friendly.args.FriendlyToggleBlank = {
       	type = "header",
		order = 195,
	}
	
    Epeen.Options.args.settings.args.Friendly.args.FriendlyToggle = {
    	name = L["Toggle"],
        desc = L["Toggle the Module On and Off"],
		order = 200,
        type = "toggle",
        get = function() return self.db.profile.on end,
        set = function() self.db.profile.on = Epeen:ToggleModuleActive("Alert") end
    }
	
	local FriendlyName = nil;

end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function EpeenFriendly:OnEnable()

end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function EpeenFriendly:ResetDB()
	copyTable(self.db.profile, EpeenFriendly.defaultDB)
end

--- ========================================================= ---
--- Toggle Friendly Frame
--- ========================================================= ---

function EpeenFriendly:ToggleFriendlyFrame()
	ListFrame:Show();
	ListFrame_TitleText:SetText("Epeen Friendly "..FriendlyType.." List v"..Revision);
end

--- ========================================================= ---
--- Friendly Player Popup
--- ========================================================= ---

function EpeenFriendly:PlayerPopup()
	Epeen_Popup:Show();
	
	EpeenFriendlyType = 1;
	EpeenPopupType = "Friendly";
	
	EpeenLabelText:SetText("Player Friendly");
end

--- ========================================================= ---
--- Friendly Guild Popup
--- ========================================================= ---

function EpeenFriendly:GuildPopup()
	Epeen_Popup:Show();
	
	EpeenFriendlyType = 2;
	EpeenPopupType = "Friendly";
	
	EpeenLabelText:SetText("Guild Friendly");
end

--- ========================================================= ---
--- Friendly Reset Button
--- ========================================================= ---

function Epeen_Friendly_Popup_Reset()
	if EpeenFriendlyType == nil then
		return;
	end
	
	Epeen_Popup_ResetValues();
end

--- ========================================================= ---
--- Friendly Remove Button
--- ========================================================= ---

function Epeen_Friendly_Popup_Remove()
	if EpeenFriendlyType == nil then
		return;
	end
	
	field = getglobal("Epeen_Popup_Name");
	value = field:GetText();
	if( value == "" ) then
		return
	end
	
	FriendlyName = value;
	
	local FriendlyNamelc = string.gsub(string.lower(FriendlyName), "%s", "_");
	
	if EpeenFriendlyType == 1 then
		if (EpeenFriendly.db.profile.friendlyplayer[FriendlyNamelc]) then
			EpeenFriendly.db.profile.friendlyplayer[FriendlyNamelc] = nil;
		end
	else
		if (EpeenFriendly.db.profile.friendlyguild[FriendlyNamelc]) then
			EpeenFriendly.db.profile.friendlyguild[FriendlyNamelc] = nil;
		end
	end
	
	Epeen_Popup:Hide();
	
	if  EpeenLastSeen and (LastSeenFrame:IsVisible()) then
		EpeenLastSeen:Refresh();
	end
end

--- ========================================================= ---
--- Friendly Accept Button
--- ========================================================= ---

function Epeen_Friendly_Popup_Accept()
	if EpeenFriendlyType == nil then
		return;
	end

	EpeenFriendly:SaveValues();
	Epeen_Popup:Hide();
	EpeenFriendly:SavetoDB();
	
	if EpeenLastSeen and (LastSeenFrame:IsVisible()) then
		EpeenLastSeen:Refresh();
	end
end

--- ========================================================= ---
--- Friendly Cancel button
--- ========================================================= ---

function Epeen_Friendly_Popup_Cancel()
	if EpeenFriendlyType == nil then
		return;
	end
	
	Epeen_Popup:Hide();
	
	EpeenFriendlyType = nil;
	EpeenPopupType = nil;
end

--- ========================================================= ---
--- Kos Popup Reset
--- ========================================================= ---

function Epeen_Popup_ResetValues()
	
	field = getglobal("Epeen_Popup_Name");
	field:SetText("");

	field = getglobal("Epeen_Popup_Reason");
	field:SetText("");
end

--- ========================================================= ---
--- Kos Popup Load Values
--- ========================================================= ---

function Epeen_Popup_LoadValues()
	Epeen_Popup_Name:SetText("");
	Epeen_Popup_Reason:SetText("");
end

--- ========================================================= ---
--- Import Save
--- ========================================================= ---

function EpeenFriendly:SaveValues()
	
	field = getglobal("Epeen_Popup_Name");
	value = field:GetText();
	if( value == "" ) then
		return
	end
	
	FriendlyName = value;

	field = getglobal("Epeen_Popup_Reason");
	value = field:GetText();
	if( value == "" ) then
		value = "";
	end
	
	FriendlyReason = value;
	
end

--- ========================================================= ---
--- Save into DB
--- ========================================================= ---

function EpeenFriendly:SavetoDB()
	if FriendlyName == "" then
		return
	end

	local FriendlyNamelc = string.gsub(string.lower(FriendlyName), "%s", "_");
	
	if EpeenFriendlyType == 1 then
		if (EpeenFriendly.db.profile.friendlyplayer[FriendlyNamelc]) then
			table.remove(self.db.profile.friendlyplayer[FriendlyNamelc])
		end
		
		self.db.profile.friendlyplayer[FriendlyNamelc] = {};
		self.db.profile.friendlyplayer[FriendlyNamelc]["name"] = FriendlyName;
		self.db.profile.friendlyplayer[FriendlyNamelc]["reason"] = FriendlyReason;
	else
		if (EpeenFriendly.db.profile.friendlyguild[FriendlyNamelc]) then
			table.remove(self.db.profile.friendlyguild[FriendlyNamelc])
		end
		
		self.db.profile.friendlyguild[FriendlyNamelc] = {};
		self.db.profile.friendlyguild[FriendlyNamelc]["name"] = FriendlyName;
		self.db.profile.friendlyguild[FriendlyNamelc]["reason"] = FriendlyReason;
	end
	
	local ShareType = nil;
	if EpeenFriendlyType == 1 then
		ShareType = "Player Friendly"
	elseif EpeenFriendlyType == 2 then
		ShareType = "Guild Friendly"
	end
	
	if EpeenShare then
		self:TriggerEvent("EpeenShareSend", ShareType, FriendlyNamelc, "Single")
	end
	
	EpeenFriendlyType = nil;
	EpeenPopupType = nil;
	
end