﻿--[[

Name: Epeen KoS
Revision: $Revision: 27401 $
Date: $Date: 2007-02-06 23:54:05 -0500 (Tue, 06 Feb 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: A module to keep track of KoS System.
Dependencies: Ace2, Epeen

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 27401 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("KoS")

L:RegisterTranslations("enUS", function() return {
    ["KoS"] = true,
    ["Toggle keeping track of Kill On Sight Enemies"] = true,
	
	["Add Player"] = true,
	["Add Player to KoS Database"] = true,
	["Add Guild"] = true,
	["Add Guild to KoS Database"] = true,
	
    ["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
	
	["KoS"] = true,
	
	["Name"] = true,
	["Reason"] = true,
	
	["KoS Player"] = true,
	["KoS Guild"] = true,
	
	["Reset"] = true,
	["Remove"] = true,
	["Accept"] = true,
	["Cancel"] = true,
} end)

EPEEN_KOS_LABEL = L["KoS"]

EPEEN_KOS_NAME = L["Name"]
EPEEN_KOS_REASON = L["Reason"]

EPEEN_RESET = L["Reset"]
EPEEN_REMOVE = L["Remove"]
EPEEN_ACCEPT = L["Accept"]
EPEEN_CANCEL = L["Cancel"]

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---
EpeenKoS = Epeen:NewModule("KoS")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function EpeenKoS:OnInitialize()
    self.db = Epeen:AcquireDBNamespace("KoS")
	
    EpeenKoS.defaultDB = {
		kosplayer = { },
		kosguild = { },
        on = true,
    }
	
	Epeen:RegisterDefaults("KoS", "profile", opts or EpeenKoS.defaultDB or {})
	
    Epeen:ToggleModuleActive("KoS", self.db.profile.on)
	
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
	Epeen.Options.args.settings.args.KoS = {
        name = L["KoS"],
        desc = L["Toggle keeping track of Kill On Sight Enemies"],
        type = "group",
		order = 103,
        args = { },
	}
	
	Epeen.Options.args.settings.args.KoS.args.KoSHeader = {
		name = bluecolor..L["KoS"].."  |c88888888"..Revision,
		icon = "Interface\\Addons\\Epeen\\icon.tga",
		iconHeight = 14,
		iconWidth = 14,
		order = 1,
		type = "header",
    }
	
	Epeen.Options.args.settings.args.KoS.args.KoSHeaderBlank = {
		order = 2,
		type = "header",
	}
	
	Epeen.Options.args.settings.args.KoS.args.KoSPlayer = {
        name = L["Add Player"],
        desc = L["Add Player to KoS Database"],
		order = 100,
        type = "execute",
    	func = function() self.PlayerPopup() end,
	}
	
	Epeen.Options.args.settings.args.KoS.args.KoSGuild = {
        name = L["Add Guild"],
        desc = L["Add Guild to KoS Database"],
		order = 101,
        type = "execute",
    	func = function() self.GuildPopup() end,
	}
	
	Epeen.Options.args.settings.args.KoS.args.KoSToggleBlank = {
       	type = "header",
		order = 195,
	}
	
    Epeen.Options.args.settings.args.KoS.args.KoSToggle = {
    	name = L["Toggle"],
        desc = L["Toggle the Module On and Off"],
		order = 200,
        type = "toggle",
        get = function() return self.db.profile.on end,
        set = function() self.db.profile.on = Epeen:ToggleModuleActive("Alert") end
    }
	
	local KoSName = nil;

end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function EpeenKoS:OnEnable()

end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function EpeenKoS:ResetDB()
	copyTable(self.db.profile, EpeenKoS.defaultDB)
end

--- ========================================================= ---
--- KoS Player Popup
--- ========================================================= ---

function EpeenKoS:PlayerPopup()
	Epeen_Popup:Show();
	
	EpeenKoSType = 1;
	EpeenPopupType = "KoS"
	
	EpeenLabelText:SetText("Player KoS");
end

--- ========================================================= ---
--- KoS Guild Popup
--- ========================================================= ---

function EpeenKoS:GuildPopup()
	Epeen_Popup:Show();
	
	EpeenKoSType = 2;
	EpeenPopupType = "KoS"
	
	EpeenLabelText:SetText("Guild KoS");
end

--- ========================================================= ---
--- KoS Reset Button
--- ========================================================= ---

function Epeen_KoS_Popup_Reset()
	if EpeenKoSType == nil then
		return;
	end
	
	Epeen_Popup_ResetValues();
end

--- ========================================================= ---
--- KoS Remove Button
--- ========================================================= ---

function Epeen_KoS_Popup_Remove()
	if EpeenKoSType == nil then
		return;
	end
	
	field = getglobal("Epeen_Popup_Name");
	value = field:GetText();
	if( value == "" ) then
		return
	end
	
	KoSName = value;
	
	local KoSNamelc = string.gsub(string.lower(KoSName), "%s", "_");
	
	if EpeenKoSType == 1 then
		if (EpeenKoS.db.profile.kosplayer[KoSNamelc]) then
			EpeenKoS.db.profile.kosplayer[KoSNamelc] = nil;
		end
	else
		if (EpeenKoS.db.profile.kosguild[KoSNamelc]) then
			EpeenKoS.db.profile.kosguild[KoSNamelc] = nil;
		end
	end
	
	Epeen_Popup:Hide();
	
	if  EpeenLastSeen and (LastSeenFrame:IsVisible()) then
		EpeenLastSeen:Refresh();
	end
end

--- ========================================================= ---
--- KoS Accept Button
--- ========================================================= ---

function Epeen_KoS_Popup_Accept()
	if EpeenKoSType == nil then
		return;
	end

	EpeenKoS:SaveValues();
	Epeen_Popup:Hide();
	EpeenKoS:SavetoDB();
	
	if EpeenLastSeen and (LastSeenFrame:IsVisible()) then
		EpeenLastSeen:Refresh();
	end
end

--- ========================================================= ---
--- KoS Cancel button
--- ========================================================= ---

function Epeen_KoS_Popup_Cancel()
	if EpeenKoSType == nil then
		return;
	end
	
	Epeen_Popup:Hide();
	
	EpeenKoSType = nil;
	EpeenPopupType = nil;
end

--- ========================================================= ---
--- Kos Popup Reset
--- ========================================================= ---

function Epeen_Popup_ResetValues()
	
	field = getglobal("Epeen_Popup_Name");
	field:SetText("");

	field = getglobal("Epeen_Popup_Reason");
	field:SetText("");
end

--- ========================================================= ---
--- Kos Popup Load Values
--- ========================================================= ---

function Epeen_Popup_LoadValues()
	Epeen_Popup_Name:SetText("");
	Epeen_Popup_Reason:SetText("");
end

--- ========================================================= ---
--- Import Save
--- ========================================================= ---

function EpeenKoS:SaveValues()
	
	field = getglobal("Epeen_Popup_Name");
	value = field:GetText();
	if( value == "" ) then
		return
	end
	
	KoSName = value;

	field = getglobal("Epeen_Popup_Reason");
	value = field:GetText();
	if( value == "" ) then
		value = "";
	end
	
	KoSReason = value;
	
end

--- ========================================================= ---
--- Save into DB
--- ========================================================= ---

function EpeenKoS:SavetoDB()
	if KoSName == "" then
		return
	end

	local KoSNamelc = string.gsub(string.lower(KoSName), "%s", "_");
	
	if EpeenKoSType == 1 then
		if (EpeenKoS.db.profile.kosplayer[KoSNamelc]) then
			table.remove(self.db.profile.kosplayer[KoSNamelc])
		end
		
		self.db.profile.kosplayer[KoSNamelc] = {};
		self.db.profile.kosplayer[KoSNamelc]["name"] = KoSName;
		self.db.profile.kosplayer[KoSNamelc]["reason"] = KoSReason;
	else
		if (EpeenKoS.db.profile.kosguild[KoSNamelc]) then
			table.remove(self.db.profile.kosguild[KoSNamelc])
		end
		
		self.db.profile.kosguild[KoSNamelc] = {};
		self.db.profile.kosguild[KoSNamelc]["name"] = KoSName;
		self.db.profile.kosguild[KoSNamelc]["reason"] = KoSReason;
	end
	
	local ShareType = nil;
	if EpeenKoSType == 1 then
		ShareType = "Player KoS"
	elseif EpeenKoSType == 2 then
		ShareType = "Guild KoS"
	end
	
	if EpeenShare then
		self:TriggerEvent("EpeenShareSend", ShareType, KoSNamelc, "Single")
	end
	
	EpeenKoSType = nil;
	EpeenPopupType = nil;
end