﻿--[[

Name: Epeen Last Seen
Revision: $Revision: 27603 $
Date: $Date: 2007-02-08 23:47:51 -0500 (Thu, 08 Feb 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: A module to keep track of enemies.
Dependencies: Ace2, Epeen

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 27603 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("LastSeen")

L:RegisterTranslations("enUS", function() return {
    ["Last Seen"] = true,
    ["Toggle keeping track of last time you have seen a player"] = true,
	
	["Track Enemy Players"] = true,
	["Toggle Tracking Enemy Players"] = true,
	["Track Friendly Players"] = true,
	["Toggle Tracking Friendly Players"] = true,
	
    ["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
	
	["Show Last Seen Frame"] = true,
    ["Toggle the Last Seen Frame"] = true,

} end)

Epeen_DropdownList = {
	{ id = 1, name = "Name", sortType = "name" },
	{ id = 2, name = "Level", sortType = "level" },
   	{ id = 3, name = "Class", sortType = "class" },
	{ id = 4, name = "Guild", sortType = "guild" },
	{ id = 5, name = "Last Seen", sortType = "lastseen" },    
};

Epeen_FactionDropdownList = {
	{ id = 1, name = "All", sortType = "all" },
	{ id = 2, name = "Enemy", sortType = "enemy" },
   	{ id = 3, name = "Friendly", sortType = "friendly" },  
};

Epeen_ListDropdownList = {
	{ id = 1, name = "Last Seen", sortType = "lastseen" },
	{ id = 2, name = "Player KoS", sortType = "playerkos" },
   	{ id = 3, name = "Player Friendly", sortType = "playerfriendly" }, 
	{ id = 4, name = "Guild KoS", sortType = "guildkos" }, 
	{ id = 5, name = "Guild Friendly", sortType = "guildfriendly" },  
};

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---

EpeenLastSeen = Epeen:NewModule("LastSeen")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function EpeenLastSeen:OnInitialize()
    self.db = Epeen:AcquireDBNamespace("LastSeen")
	
    EpeenLastSeen.defaultDB = {
        on = true,
		trackenemies = true,
		trackfriendly = true,
		chars = { },
		guilds = { },
		currentsort = {1, "name"},
		currentfaction = {1, "all"},
		currentlist = {1, "lastseen"},
    }
	
	Epeen:RegisterDefaults("LastSeen", "profile", opts or EpeenLastSeen.defaultDB or {})
	
    Epeen:ToggleModuleActive("LastSeen", self.db.profile.on)
  
	
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
	Epeen.Options.args.settings.args.LastSeen = {
        name = L["Last Seen"],
        desc = L["Toggle keeping track of last time you have seen a player"],
        type = "group",
		order = 104,
        args = { },
	}
	
	Epeen.Options.args.settings.args.LastSeen.args.LastSeenHeader = {
		name = bluecolor..L["Last Seen"].."  |c88888888"..Revision,
		icon = "Interface\\Addons\\Epeen\\icon.tga",
		iconHeight = 14,
		iconWidth = 14,
		order = 1,
		type = "header",
    }
	
	Epeen.Options.args.settings.args.LastSeen.args.LastSeenHeaderBlank = {
		order = 2,
		type = "header",
	}
	
	Epeen.Options.args.settings.args.LastSeen.args.LastSeenTrackEnemy = {
        name = L["Track Enemy Players"],
        desc = L["Toggle Tracking Enemy Players"],
		order = 100,
        type = "toggle",
        get = function() return self.db.profile.trackenemies end,
        set = function() self.db.profile.trackenemies = not(self.db.profile.trackenemies) end
	}
	
	Epeen.Options.args.settings.args.LastSeen.args.LastSeenTrackFriendly = {
        name = L["Track Friendly Players"],
        desc = L["Toggle Tracking Friendly Players"],
		order = 101,
        type = "toggle",
         get = function() return self.db.profile.trackfriendly end,
         set = function() self.db.profile.trackfriendly = not(self.db.profile.trackfriendly) end
	}
	
	Epeen.Options.args.settings.args.LastSeen.args.LastSeenToggleBlank = {
		order = 195,
		type = "header",
	}
	
    Epeen.Options.args.settings.args.LastSeen.args.LastSeenToggle = {
    	name = L["Toggle"],
        desc = L["Toggle the Module On and Off"],
		order = 200,
        type = "toggle",
        get = function() return self.db.profile.on end,
        set = function() self.db.profile.on = Epeen:ToggleModuleActive("Alert") end
    }
	
	Epeen.Options.args.LastSeenFrameToggle = {
        name = L["Show Last Seen Frame"],
        desc = L["Toggle the Last Seen Frame"],
		order = 10,
        type = "execute",
		func = function() self:ToggleLastSeenFrame() end,
	}

	Epeen_TimeOffSet = 1112600000;

end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function EpeenLastSeen:OnEnable()
	self:RegisterEvent("EpeenSeen");
end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function EpeenLastSeen:ResetDB()
	copyTable(self.db.profile, EpeenLastSeen.defaultDB)
end

--- ========================================================= ---
--- Seen
--- ========================================================= ---
function EpeenLastSeen:EpeenSeen(SeenFaction, SeenPlayerName, SeenServer, SeenLevel, SeenRace, SeenClass, SeenGuild, SeenGuildRank)
	
	local playerFaction = UnitFactionGroup("player");
	
	if self.db.profile.trackenemies and playerFaction ~= SeenFaction then
	
		if self.db.profile.chars[SeenPlayerName] then
			if not(self.db.profile.chars[SeenPlayerName].server) and SeenServer then
				return;
			end
		end
		
		self.db.profile.chars[SeenPlayerName] = {};
		self.db.profile.chars[SeenPlayerName]["faction"] = SeenFaction;
		self.db.profile.chars[SeenPlayerName]["name"] = SeenPlayerName;
		self.db.profile.chars[SeenPlayerName]["server"] = SeenServer;
		self.db.profile.chars[SeenPlayerName]["level"] = SeenLevel;
		self.db.profile.chars[SeenPlayerName]["race"] = SeenRace;
		self.db.profile.chars[SeenPlayerName]["class"] = SeenClass;
		self.db.profile.chars[SeenPlayerName]["guild"] = SeenGuild;
		self.db.profile.chars[SeenPlayerName]["guildrank"] = SeenGuildRank;
		self.db.profile.chars[SeenPlayerName]["lastseen"] = time() - Epeen_TimeOffSet;
	end

	if self.db.profile.trackfriendly and playerFaction == SeenFaction then
	
		if self.db.profile.chars[SeenPlayerName] then
			if not(self.db.profile.chars[SeenPlayerName].server) and SeenServer then
				return;
			end
		end
	
		self.db.profile.chars[SeenPlayerName] = {};
		self.db.profile.chars[SeenPlayerName]["faction"] = SeenFaction;
		self.db.profile.chars[SeenPlayerName]["name"] = SeenPlayerName;
		self.db.profile.chars[SeenPlayerName]["server"] = SeenServer;
		self.db.profile.chars[SeenPlayerName]["level"] = SeenLevel;
		self.db.profile.chars[SeenPlayerName]["race"] = SeenRace;
		self.db.profile.chars[SeenPlayerName]["class"] = SeenClass;
		self.db.profile.chars[SeenPlayerName]["guild"] = SeenGuild;
		self.db.profile.chars[SeenPlayerName]["guildrank"] = SeenGuildRank;
		self.db.profile.chars[SeenPlayerName]["lastseen"] = time() - Epeen_TimeOffSet;
	end
end

--- ========================================================= ---
--- Toggle Last Seen Frame
--- ========================================================= ---

function EpeenLastSeen:ToggleLastSeenFrame()
	if (LastSeenFrame:IsVisible()) then
		LastSeenFrame:Hide();
	else
		LastSeenFrame:Show();
		
		LastSeenFrame_TitleText:SetText("Epeen Last Seen v"..Revision);
	end
end

--- ========================================================= ---
--- Epeen CapitalizeWords
--- ========================================================= ---

Epeen_CapitalizeWords = function ( phrase )
	local phrase = string.gsub(phrase, "(_)"," ") 
	local words = Epeen_Split(phrase, " ");
	local capitalizedPhrase = "";

	for i=1,words.n do 
		local v = words[i];
		if ( i ~= 1 ) then
			capitalizedPhrase = capitalizedPhrase.." ";
		end
		capitalizedPhrase = capitalizedPhrase..string.upper(string.sub(v,1,1))..string.sub(v,2);
	end

	return capitalizedPhrase;
end;

--- ========================================================= ---
--- Epeen Split
--- ========================================================= ---

Epeen_Split = function ( text, separator )
	local t = {};
	t.n = 0;
	for value in string.gmatch(text,"[^"..separator.."]+") do
		t.n = t.n + 1;
		t[t.n] = value;
	end
	return t;
end;

--- ========================================================= ---
--- Epeen Time to String
--- ========================================================= ---

function EpeenLastSeen:TimeToString(time)
   local ret = "";
   local units = 0;

   if( time < 0 ) then
      return "";
   end

   local days = floor( time / 86400 );

   if( days > 0 ) then
      time = time - days * 86400;
      ret = days .. "d ";
      units = units + 1;
   end

   local hours = floor( time / 3600 );
   if( hours > 0 ) then
      time = time - hours * 3600;
      ret = ret .. hours .. "h ";
      units = units + 1;
   end

   if( units == 2 ) then
      return ret;
   end

   local minutes = floor ( time / 60 );
   if( minutes > 0 ) then
      time = time - minutes * 60;
      ret = ret .. minutes .. "m ";
      units = units + 1;
   end

   if( units == 2 ) then
      return ret;
   end

   return ret .. time .. "s";
end

--- ========================================================= ---
--- Epeen Faction DropDown Initialize
--- ========================================================= ---

function EpeenLastSeen:FactionDropDown_Initialize()
	local info;
	for i = 1, getn(Epeen_FactionDropdownList), 1 do
		info = { };
		info.text = Epeen_FactionDropdownList[i].name;
		info.func = EpeenLastSeen.FactionDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

--- ========================================================= ---
--- Epeen Faction DropDown OnLoad
--- ========================================================= ---

function EpeenLastSeen:FactionDropDown_OnLoad()
	UIDropDownMenu_Initialize(LastSeenFrame_FactionDropDown, EpeenLastSeen.FactionDropDown_Initialize);
    EpeenLastSeen:FactionDropDown_SetSelectedID(LastSeenFrame_FactionDropDown, 1, Epeen_FactionDropdownList);
	UIDropDownMenu_SetWidth(80);
	UIDropDownMenu_SetButtonWidth(24);
	UIDropDownMenu_JustifyText("LEFT", LastSeenFrame_FactionDropDown)
end

--- ========================================================= ---
--- Epeen Faction DropDown OnShow
--- ========================================================= ---

function EpeenLastSeen:FactionDropDown_OnShow()
    EpeenLastSeen:FactionDropDown_SetSelectedID(LastSeenFrame_FactionDropDown, EpeenLastSeen.db.profile.currentfaction[1], Epeen_FactionDropdownList);
end

--- ========================================================= ---
--- Epeen Faction DropDown OnClick
--- ========================================================= ---

function EpeenLastSeen:FactionDropDown_OnClick()
   local oldID = UIDropDownMenu_GetSelectedID(LastSeenFrame_FactionDropDown);
	UIDropDownMenu_SetSelectedID(LastSeenFrame_FactionDropDown, this:GetID());
	if( oldID ~= this:GetID() ) then
	   EpeenLastSeen.db.profile.currentfaction[1] = Epeen_FactionDropdownList[this:GetID()].id
	   EpeenLastSeen.db.profile.currentfaction[2] = Epeen_FactionDropdownList[this:GetID()].sortType;
   	   EpeenLastSeen:Refresh();
	end
end

--- ========================================================= ---
--- Epeen Faction DropDown SetSelected
--- ========================================================= ---

function EpeenLastSeen:FactionDropDown_SetSelectedID(frame, id, names)
	UIDropDownMenu_SetSelectedID(frame, id);
	if( not frame ) then
		frame = this;
	end
	UIDropDownMenu_SetText(names[id].name, frame);
end

--- ========================================================= ---
--- Epeen DropDown Initialize
--- ========================================================= ---

function EpeenLastSeen:DropDown_Initialize()
	local info;
	for i = 1, getn(Epeen_DropdownList), 1 do
		info = { };
		info.text = Epeen_DropdownList[i].name;
		info.func = EpeenLastSeen.DropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

--- ========================================================= ---
--- Epeen DropDown OnLoad
--- ========================================================= ---

function EpeenLastSeen:DropDown_OnLoad()
	UIDropDownMenu_Initialize(LastSeenFrame_DropDown, EpeenLastSeen.DropDown_Initialize);
    EpeenLastSeen:DropDown_SetSelectedID(LastSeenFrame_DropDown, 1, Epeen_DropdownList);
	UIDropDownMenu_SetWidth(80);
	UIDropDownMenu_SetButtonWidth(24);
	UIDropDownMenu_JustifyText("LEFT", LastSeenFrame_DropDown)
end

--- ========================================================= ---
--- Epeen DropDown OnShow
--- ========================================================= ---

function EpeenLastSeen:DropDown_OnShow()
    EpeenLastSeen:DropDown_SetSelectedID(LastSeenFrame_DropDown, EpeenLastSeen.db.profile.currentsort[1], Epeen_DropdownList);
end

--- ========================================================= ---
--- Epeen DropDown OnClick
--- ========================================================= ---

function EpeenLastSeen:DropDown_OnClick()
   local oldID = UIDropDownMenu_GetSelectedID(LastSeenFrame_DropDown);
	UIDropDownMenu_SetSelectedID(LastSeenFrame_DropDown, this:GetID());
	if( oldID ~= this:GetID() ) then
		EpeenLastSeen.db.profile.currentsort[1] = Epeen_DropdownList[this:GetID()].id
		EpeenLastSeen.db.profile.currentsort[2] = Epeen_DropdownList[this:GetID()].sortType;
   		EpeenLastSeen:Refresh();
	end
end

--- ========================================================= ---
--- Epeen DropDown SetSelected
--- ========================================================= ---

function EpeenLastSeen:DropDown_SetSelectedID(frame, id, names)
	UIDropDownMenu_SetSelectedID(frame, id);
	if( not frame ) then
		frame = this;
	end
	UIDropDownMenu_SetText(names[id].name, frame);
end

--- ========================================================= ---
--- Epeen List DropDown Initialize
--- ========================================================= ---

function EpeenLastSeen:ListDropDown_Initialize()
	local info;
	for i = 1, getn(Epeen_ListDropdownList), 1 do
		info = { };
		info.text = Epeen_ListDropdownList[i].name;
		info.func = EpeenLastSeen.ListDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

--- ========================================================= ---
--- Epeen List DropDown OnLoad
--- ========================================================= ---

function EpeenLastSeen:ListDropDown_OnLoad()
	UIDropDownMenu_Initialize(LastSeenFrame_ListDropDown, EpeenLastSeen.ListDropDown_Initialize);
    EpeenLastSeen:ListDropDown_SetSelectedID(LastSeenFrame_ListDropDown, 1, Epeen_ListDropdownList);
	UIDropDownMenu_SetWidth(100);
	UIDropDownMenu_SetButtonWidth(24);
	UIDropDownMenu_JustifyText("LEFT", LastSeenFrame_ListDropDown)
end

--- ========================================================= ---
--- Epeen List DropDown OnShow
--- ========================================================= ---

function EpeenLastSeen:ListDropDown_OnShow()
    EpeenLastSeen:ListDropDown_SetSelectedID(LastSeenFrame_ListDropDown, EpeenLastSeen.db.profile.currentlist[1], Epeen_ListDropdownList);
end

--- ========================================================= ---
--- Epeen List DropDown OnClick
--- ========================================================= ---

function EpeenLastSeen:ListDropDown_OnClick()
   local oldID = UIDropDownMenu_GetSelectedID(LastSeenFrame_ListDropDown);
	UIDropDownMenu_SetSelectedID(LastSeenFrame_ListDropDown, this:GetID());
	if( oldID ~= this:GetID() ) then
	   EpeenLastSeen.db.profile.currentlist[1] = Epeen_ListDropdownList[this:GetID()].id
	   EpeenLastSeen.db.profile.currentlist[2] = Epeen_ListDropdownList[this:GetID()].sortType;
   	   EpeenLastSeen:Refresh();
	end
end

--- ========================================================= ---
--- Epeen List DropDown SetSelected
--- ========================================================= ---

function EpeenLastSeen:ListDropDown_SetSelectedID(frame, id, names)
	UIDropDownMenu_SetSelectedID(frame, id);
	if( not frame ) then
		frame = this;
	end
	UIDropDownMenu_SetText(names[id].name, frame);
end

--- ========================================================= ---
--- Epeen Item Button (On Click)
--- ========================================================= ---

function EpeenLastSeen:ItemButton_OnClick(arg1)
	if EpeenLastSeen.db.profile.currentlist[2] == "playerkos" and EpeenKoS then
		local itemid = this:GetID();
		local newitemname = getglobal("LastSeenFrame_Item" .. itemid .. "Name");
		local value = newitemname:GetText();
			
		Epeen_Popup:Show();
		EpeenKoSType = 1;
		EpeenPopupType = "KoS";
		EpeenLabelText:SetText("Player KoS");
		Epeen_Popup_Name:SetText(value);
		
		valuelc = string.gsub(string.lower(value), "%s", "_");
		if EpeenKoS.db.profile.kosplayer[valuelc].reason and EpeenKoS then
			Epeen_Popup_Reason:SetText(EpeenKoS.db.profile.kosplayer[valuelc].reason);
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "guildkos" and EpeenKoS then
		local itemid = this:GetID();
		local newitemname = getglobal("LastSeenFrame_Item" .. itemid .. "Name");
		local value = newitemname:GetText();
			
		Epeen_Popup:Show();
		EpeenKoSType = 2;
		EpeenPopupType = "KoS";
		EpeenLabelText:SetText("Guild KoS");
		Epeen_Popup_Name:SetText(value);
		
		valuelc = string.gsub(string.lower(value), "%s", "_");
		if EpeenKoS.db.profile.kosguild[valuelc].reason and EpeenKoS then
			Epeen_Popup_Reason:SetText(EpeenKoS.db.profile.kosguild[valuelc].reason);
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "playerfriendly" and EpeenFriendly then
		local itemid = this:GetID();
		local newitemname = getglobal("LastSeenFrame_Item" .. itemid .. "Name");
		local value = newitemname:GetText();
			
		Epeen_Popup:Show();
		EpeenFriendlyType = 1;
		EpeenPopupType = "Friendly";
		EpeenLabelText:SetText("Player Friendly");
		Epeen_Popup_Name:SetText(value);
		
		valuelc = string.gsub(string.lower(value), "%s", "_");
		if EpeenFriendly.db.profile.friendlyplayer[valuelc].reason and EpeenFriendly then
			Epeen_Popup_Reason:SetText(EpeenFriendly.db.profile.friendlyplayer[valuelc].reason);
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "guildfriendly" and EpeenFriendly then
		local itemid = this:GetID();
		local newitemname = getglobal("LastSeenFrame_Item" .. itemid .. "Name");
		local value = newitemname:GetText();
			
		Epeen_Popup:Show();
		EpeenFriendlyType = 2;
		EpeenPopupType = "Friendly";
		EpeenLabelText:SetText("Guild Friendly");
		Epeen_Popup_Name:SetText(value);
		
		valuelc = string.gsub(string.lower(value), "%s", "_");
		if EpeenFriendly.db.profile.friendlyguild[valuelc].reason and EpeenFriendly then
			Epeen_Popup_Reason:SetText(EpeenFriendly.db.profile.friendlyguild[valuelc].reason);
		end
	end
end

--- ========================================================= ---
--- Epeen Item Button (On Enter)
--- ========================================================= ---

function EpeenLastSeen:ItemButton_OnEnter()
   local buttonID = this:GetID();
   local name = LastSeenFrame_Items[buttonID + FauxScrollFrame_GetOffset(LastSeenFrame_ScrollFrame) ];
   EpeenLastSeen:PlayerListTooltip(name);
end

--- ========================================================= ---
--- Epeen Player List Tooltip
--- ========================================================= ---

function EpeenLastSeen:PlayerListTooltip(name)
	local currentPlayer = nil;

	local namelc = nil;
	if name ~= nil then
		namelc = string.gsub(string.lower(name), "%s", "_");
	end

	if EpeenLastSeen.db.profile.currentlist[2] == "lastseen" then
    	if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)] then
			currentPlayer = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)];
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "playerkos" and EpeenKoS then
		if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)] then
			currentPlayer = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)];
		else
			currentPlayer = EpeenKoS.db.profile.kosplayer[namelc];
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "guildkos" and EpeenKoS then
		currentPlayer = EpeenKoS.db.profile.kosguild[namelc];
	elseif EpeenLastSeen.db.profile.currentlist[2] == "playerfriendly" and EpeenFriendly then
		if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)] then
			currentPlayer = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)];
		else
			currentPlayer = EpeenFriendly.db.profile.friendlyplayer[namelc];
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "guildfriendly" and EpeenFriendly then
		currentPlayer = EpeenFriendly.db.profile.friendlyguild[namelc];
	end

	if not currentPlayer then
		return;
	end
   
	local tooltip = nil;

	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
   
	tooltip = "|c00ffff00";
	tooltip = tooltip .. Epeen_CapitalizeWords(currentPlayer.name) .. "\n";
	tooltip = tooltip .. "|cffffffff";
	
	if EpeenLastSeen.db.profile.currentlist[2] == "lastseen" or EpeenLastSeen.db.profile.currentlist[2] == "playerkos" or EpeenLastSeen.db.profile.currentlist[2] == "playerfriendly" then

		if (currentPlayer.level) then
			tooltip = tooltip .. currentPlayer.level .. " ";
		end

		if (currentPlayer.race) then
			tooltip = tooltip .. currentPlayer.race .. " ";
		end

		if (currentPlayer.class) then
			tooltip = tooltip .. currentPlayer.class .. "";
		end

		tooltip = tooltip .. "\n";
	
		if (currentPlayer.guildrank) and (currentPlayer.guildrank ~= "") then
			tooltip = tooltip .. currentPlayer.guildrank .. " of ";
		end
	
		if (currentPlayer.guild) then
			tooltip = tooltip .. currentPlayer.guild .. "";
		end
		
		local playerlc = string.lower(currentPlayer.name);
		local guildnamelc = nil;
	
		if (currentPlayer.guild) then
			guildnamelc = string.gsub(string.lower(currentPlayer.guild), "%s", "_");
		end

		if (EpeenKoS.db.profile.kosplayer[playerlc]) then
			tooltip = tooltip .. "\n";
			tooltip = tooltip .. "Player KoS";
			if (EpeenKoS.db.profile.kosplayer[playerlc].reason) then
				tooltip = tooltip .. ": " .. EpeenKoS.db.profile.kosplayer[playerlc].reason .. "";
			end
		elseif (EpeenFriendly.db.profile.friendlyplayer[playerlc]) then
			tooltip = tooltip .. "Player Friendly";
			if (EpeenFriendly.db.profile.friendlyplayer[playerlc].reason) then
				tooltip = tooltip .. ": " .. EpeenFriendly.db.profile.friendlyplayer[playerlc].reason .. "";
			end
		end
	
		if (currentPlayer.guild) then
			if (EpeenKoS.db.profile.kosguild[guildnamelc]) then
				tooltip = tooltip .. "\n";
				tooltip = tooltip .. "Guild KoS";
				if (EpeenKoS.db.profile.kosguild[guildnamelc].reason) then
					tooltip = tooltip .. ": " .. EpeenKoS.db.profile.kosguild[guildnamelc].reason .. "";
				end
			end
		elseif (currentPlayer.guild) then
			if (EpeenFriendly.db.profile.friendlyguild[guildnamelc]) then
				if (EpeenFriendly.db.profile.friendlyplayer[playerlc]) then
					tooltip = tooltip .. "\n";
				end
				tooltip = tooltip .. "Guild Friendly";
				if (EpeenFriendly.db.profile.friendlyguild[guildnamelc].reason) then
					tooltip = tooltip .. ": " .. EpeenFriendly.db.profile.friendlyguild[guildnamelc].reason .. "";
				end
			end
		end

		if EpeenStats then
			if EpeenStats.db.profile.chars[Epeen_CapitalizeWords(currentPlayer.name)] then
				tooltip = tooltip .. "\n";
				tooltip = tooltip .. "Kills/Deaths: ("..EpeenStats.db.profile.chars[Epeen_CapitalizeWords(currentPlayer.name)].win.."/"..EpeenStats.db.profile.chars[Epeen_CapitalizeWords(currentPlayer.name)].loss..")";
			end
		end

		tooltip = tooltip .. "\n";

		if (currentPlayer.lastseen) then
			local timeNow = time() - Epeen_TimeOffSet;
			tooltip = tooltip .. "Last Seen: " .. EpeenLastSeen:TimeToString(timeNow - currentPlayer.lastseen) .. "";
		end

		GameTooltip:SetText(tooltip);

		local playerFaction = UnitFactionGroup("player");
	
		if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)] then
			if (currentPlayer.faction == playerFaction ) then
				GameTooltip:SetBackdropColor(0, 0, 0.5);
    		else
				GameTooltip:SetBackdropColor(0.5, 0, 0);
    		end
		else
			GameTooltip:SetBackdropColor(0.5, 0.5, 0);
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "guildkos" then
		local guildwin = 0;
		local guildloss = 0;
	
		if (EpeenKoS.db.profile.kosguild[namelc]) then
			tooltip = tooltip .. "Guild KoS";
			if (EpeenKoS.db.profile.kosguild[namelc].reason) then
				tooltip = tooltip .. ": " .. EpeenKoS.db.profile.kosguild[namelc].reason .. "";
			end
		end
		
		tooltip = tooltip .. "\n";
		
		local guildcount = 0;
		for i,data in pairs(EpeenLastSeen.db.profile.chars) do
			if data.guild then
				local tmp_guild = string.gsub(string.lower(data.guild), "%s", "_");
				if tmp_guild == namelc then
					guildcount = guildcount + 1;
				end
			end
		end
		
		tooltip = tooltip .. "Members Stored: "..guildcount;
		
		if EpeenStats then
			for i,data in pairs(EpeenStats.db.profile.chars) do
				if EpeenLastSeen.db.profile.chars[data.name] then
					if EpeenLastSeen.db.profile.chars[data.name].guild then
						local tmp_guild = string.gsub(string.lower(EpeenLastSeen.db.profile.chars[data.name].guild), "%s", "_");
						if tmp_guild == namelc then
							guildwin = guildwin + data.win;
							guildloss = guildloss + data.loss;
						end
					end
				end
			end
		end
		
		if guildwin ~= 0 or guildloss ~= 0 then
			tooltip = tooltip .. "\n";
			tooltip = tooltip .. "Kills: "..guildwin;
			tooltip = tooltip .. "\n";
			tooltip = tooltip .. "Deaths: "..guildloss;
		end
		
		GameTooltip:SetText(tooltip);
		
		GameTooltip:SetBackdropColor(0.5, 0, 0);
		
	elseif EpeenLastSeen.db.profile.currentlist[2] == "guildfriendly" then
		local guildkill = 0;
		local guildloss = 0;
		
		if (EpeenFriendly.db.profile.friendlyguild[namelc]) then
			tooltip = tooltip .. "Guild Friendly";
			if (EpeenFriendly.db.profile.friendlyguild[namelc].reason) then
				tooltip = tooltip .. ": " .. EpeenFriendly.db.profile.friendlyguild[namelc].reason .. "";
			end
		end
		
		tooltip = tooltip .. "\n";
		
		local guildcount = 0;
		for i,data in pairs(EpeenLastSeen.db.profile.chars) do
			if data.guild then
				local tmp_guild = string.gsub(string.lower(data.guild), "%s", "_");
				if tmp_guild == namelc then
					guildcount = guildcount + 1;
				end
			end
		end
		
		tooltip = tooltip .. "Members Stored: "..guildcount;
		
		GameTooltip:SetText(tooltip);
		
		GameTooltip:SetBackdropColor(0, 0, 0.5);
	end

	GameTooltip:Show();
  
end

--- ========================================================= ---
--- Epeen Item Button (On Leave)
--- ========================================================= ---

function EpeenLastSeen:ItemButton_OnLeave()
   GameTooltip:Hide();
end

--- ========================================================= ---
--- Epeen Refresh
--- ========================================================= ---

function EpeenLastSeen:Refresh()
   FauxScrollFrame_SetOffset(LastSeenFrame_ScrollFrame, 0);
   getglobal("LastSeenFrame_ScrollFrameScrollBar"):SetValue(0);
   EpeenLastSeen:BuildDisplayItems();
   EpeenLastSeen:Update();
end

--- ========================================================= ---
--- Epeen Build Items
--- ========================================================= ---

function EpeenLastSeen:BuildDisplayItems()
	local iNew = 1;
	local clear = false;

	if EpeenLastSeen.db.profile.currentlist[2] == "lastseen" then
    	if (EpeenLastSeen.db.profile.chars == nil) then
	   		return;
		end
	end
	
	if EpeenLastSeen.db.profile.currentlist[2] == "playerkos" then
    	if (EpeenKoS.db.profile.kosplayer == nil) then
	   		return;
		end
	end
	
	if EpeenLastSeen.db.profile.currentlist[2] == "guildkos" then
    	if (EpeenKoS.db.profile.kosguild == nil) then
	   		return;
		end
	end
	
	if EpeenLastSeen.db.profile.currentlist[2] == "playerfriendly" then
    	if (EpeenFriendly.db.profile.friendlyplayer == nil) then
	   		return;
		end
	end
	
	if EpeenLastSeen.db.profile.currentlist[2] == "guildfriendly" then
    	if (EpeenFriendly.db.profile.friendlyguild == nil) then
	   		return;
		end
	end

    FauxScrollFrame_SetOffset(LastSeenFrame_ScrollFrame, 0);
	getglobal("LastSeenFrame_ScrollFrameScrollBar"):SetValue(0);
	
	local playerFaction = UnitFactionGroup("player");
	
	sorted = { }
	LastSeenFrame_Items = { };
	
	if EpeenLastSeen.db.profile.currentlist[2] == "lastseen" then
		for k,v in pairs(EpeenLastSeen.db.profile.chars) do
			if EpeenLastSeen.db.profile.currentfaction[2] == "all" and v.server == nil then
    			table.insert(sorted, v)
			elseif EpeenLastSeen.db.profile.currentfaction[2] == "friendly" and v.server == nil then
				if v.faction == playerFaction then
					table.insert(sorted, v)
				end
			elseif EpeenLastSeen.db.profile.currentfaction[2] == "enemy" and v.server == nil then
				if v.faction ~= playerFaction then
					table.insert(sorted, v)
				end
			end
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "playerkos" then
		if EpeenKoS then
			clear = true;
			for k,v in pairs(EpeenKoS.db.profile.kosplayer) do
				table.insert(sorted, v)
			end
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "guildkos" then
		if EpeenKoS then
			for k,v in pairs(EpeenKoS.db.profile.kosguild) do
				table.insert(sorted, v)
			end
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "playerfriendly" then
		if EpeenFriendly then
			clear = true;
			for k,v in pairs(EpeenFriendly.db.profile.friendlyplayer) do
				table.insert(sorted, v)
			end
		end
	elseif EpeenLastSeen.db.profile.currentlist[2] == "guildfriendly" then
		if EpeenFriendly then
			for k,v in pairs(EpeenFriendly.db.profile.friendlyguild) do
				table.insert(sorted, v)
			end
		end
	end
	
	for i, data in pairs(sorted) do
		data.name = string.gsub(string.lower(data.name), "%s", "_");
		
		if (EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(data.name)]) then
			data.level = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(data.name)].level;
			data.class = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(data.name)].class;
			data.guild = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(data.name)].guild;
			if data.guild == nil then
				data.guild = "";
			end
			data.lastseen = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(data.name)].lastseen;
		else
			if data.level == nil then
				data.level = 0;
			end
			if data.class == nil then
				data.class = "";
			end
			if data.guild == nil then
				data.guild = "";
			end
			if data.lastseen == nil then
				data.lastseen = 0;
			end
		end
	end
	
	if getn(sorted) > 1 then
		if EpeenLastSeen.db.profile.currentlist[2] == "guildkos" or EpeenLastSeen.db.profile.currentlist[2] == "guildfriendly" then
			table.sort(sorted, function (a, b) return a["name"] < b["name"] end)
		else
			if EpeenLastSeen.db.profile.currentsort[2] == "lastseen" then
				table.sort(sorted, function (a, b) return a[EpeenLastSeen.db.profile.currentsort[2]] > b[EpeenLastSeen.db.profile.currentsort[2]] end)
			else
				table.sort(sorted, function (a, b) return a[EpeenLastSeen.db.profile.currentsort[2]] < b[EpeenLastSeen.db.profile.currentsort[2]] end)
			end
		end
	end
	
	for i, data in pairs(sorted) do
    	LastSeenFrame_Items[iNew] = data.name;
		iNew = iNew + 1;
		if clear then
			data.level = nil;
			data.class = nil;
			data.guild = nil;
			data.lastseen = nil;
		end
	end

	LastSeenFrame_Items.onePastEnd = iNew;
end


--- ========================================================= ---
--- Epeen Update
--- ========================================================= ---

function EpeenLastSeen:Update()
   local iItem;

   if (LastSeenFrame_Items == nil) then
      EpeenLastSeen:BuildDisplayItems();
   end

   if (LastSeenFrame_Items == nil) then
      return;
   end

   FauxScrollFrame_Update(LastSeenFrame_ScrollFrame, LastSeenFrame_Items.onePastEnd - 1, 23, 16);

   local playerFaction = UnitFactionGroup("player");
   local timeNow = time() - Epeen_TimeOffSet;

	for iItem = 1, 23, 1 do
		local itemIndex = iItem + FauxScrollFrame_GetOffset(LastSeenFrame_ScrollFrame);
		local playerDBItem = getglobal("LastSeenFrame_Item"..iItem);
        local name = LastSeenFrame_Items[itemIndex];
        local currentPlayer = nil;
		
		local namelc = nil;
		if LastSeenFrame_Items[itemIndex] ~= nil then
			namelc = string.gsub(string.lower(LastSeenFrame_Items[itemIndex]), "%s", "_");
		end

		if( itemIndex < LastSeenFrame_Items.onePastEnd ) then
			if EpeenLastSeen.db.profile.currentlist[2] == "lastseen" then
        		if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)] then
					currentPlayer = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)];
				end
			elseif EpeenLastSeen.db.profile.currentlist[2] == "playerkos" and EpeenKoS then
				if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)] then
					currentPlayer = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)];
				else
					currentPlayer = EpeenKoS.db.profile.kosplayer[namelc];
				end
			elseif EpeenLastSeen.db.profile.currentlist[2] == "guildkos" and EpeenKoS then
				currentPlayer = EpeenKoS.db.profile.kosguild[namelc];
			elseif EpeenLastSeen.db.profile.currentlist[2] == "playerfriendly" and EpeenFriendly then
				if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)] then
					currentPlayer = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)];
				else
					currentPlayer = EpeenFriendly.db.profile.friendlyplayer[namelc];
				end
			elseif EpeenLastSeen.db.profile.currentlist[2] == "guildfriendly" and EpeenFriendly then
				currentPlayer = EpeenFriendly.db.profile.friendlyguild[namelc];
			end
        end

        if (currentPlayer ~= nil) then
        	local playerDBItemName = getglobal("LastSeenFrame_Item" .. iItem .. "Name");
			playerDBItemName:SetText(Epeen_CapitalizeWords(name));
			
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(name)] then
				getglobal("LastSeenFrame_Item" .. iItem .. "Level"):SetText(currentPlayer.level);
				getglobal("LastSeenFrame_Item" .. iItem .. "Class"):SetText(currentPlayer.class);
           	 	getglobal("LastSeenFrame_Item" .. iItem .. "Guild"):SetText(currentPlayer.guild);
            	getglobal("LastSeenFrame_Item" .. iItem .. "LastSeen"):SetText(EpeenLastSeen:TimeToString(timeNow - currentPlayer.lastseen));
				
				if EpeenStats then
					if EpeenStats.db.profile.chars[Epeen_CapitalizeWords(name)] then
						local killdeath = "("..EpeenStats.db.profile.chars[Epeen_CapitalizeWords(name)].win.."/"..EpeenStats.db.profile.chars[Epeen_CapitalizeWords(name)].loss..")";
						getglobal("LastSeenFrame_Item" .. iItem .. "Kills"):SetText(killdeath);
					else
						getglobal("LastSeenFrame_Item" .. iItem .. "Kills"):SetText("");
					end
				else
					getglobal("LastSeenFrame_Item" .. iItem .. "Kills"):SetText("");
				end

            	if (currentPlayer.faction == playerFaction) then
					playerDBItemName:SetTextColor(0.4, 0.4, 1);
            	else
					playerDBItemName:SetTextColor(1, 0.4, 0.4);
            	end
			else
				getglobal("LastSeenFrame_Item" .. iItem .. "Level"):SetText("");
				getglobal("LastSeenFrame_Item" .. iItem .. "Class"):SetText("");
           	 	getglobal("LastSeenFrame_Item" .. iItem .. "Guild"):SetText("");
            	getglobal("LastSeenFrame_Item" .. iItem .. "LastSeen"):SetText("");
				getglobal("LastSeenFrame_Item" .. iItem .. "Kills"):SetText("");
				
				playerDBItemName:SetTextColor(1, 1, 1);
			end
			
			if EpeenLastSeen.db.profile.currentlist[2] == "guildkos" then
				getglobal("LastSeenFrame_Item" .. iItem .. "Guild"):SetText(currentPlayer.reason);
				
				playerDBItemName:SetTextColor(1, 0.4, 0.4);
			end
			
			if EpeenLastSeen.db.profile.currentlist[2] == "guildfriendly" then
				getglobal("LastSeenFrame_Item" .. iItem .. "Guild"):SetText(currentPlayer.reason);
				
				playerDBItemName:SetTextColor(0.4, 0.4, 1);
			end

			playerDBItem:Show();
		else
			playerDBItem:Hide();
		end
	end
end