﻿--[[

Name: Epeen Share
Revision: $Revision: 23844 $
Date: $Date: 2007-01-07 03:01:22 -0500 (Sun, 07 Jan 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: A module to Share Epeen Information.
Dependencies: Ace2, Epeen

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 23844 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("Share")

L:RegisterTranslations("enUS", function() return {
    ["Share"] = true,
    ["Toggle sharing of Epeen Information"] = true,
	
	["Shared Database"] = true,
    ["Database to share automatically"] = true,

	["Toggles sending Player KoS broadcasts"] = true,
	["Toggles sending Guild KoS broadcasts"] = true,
	["Toggles sending Player Friendly broadcasts"] = true,
	["Toggles sending Guild Friendly broadcasts"] = true,
	
	["Receive Filter"] = true,
    ["Only accept broadcast sends for these databases"] = true,
	

    ["Toggles receiving Player KoS broadcasts"] = true,
    ["Toggles receiving Guild KoS broadcasts"] = true,
    ["Toggles receiving Player Friendly broadcasts"] = true,
    ["Toggles receiving Guild Friendly broadcasts"] = true,
	
	["Player KoS"] = true,
	["Guild KoS"] = true,
	["Player Friendly"] = true,
	["Guild Friendly"] = true,
	
	["Broadcast Database"] = true,
	["Broadcast your entire database on your selected channel"] = true,
	
	["Send Player KoS Database"] = true,
	["Send Guild KoS Database"] = true,
	["Send Player Friendly Database"] = true,
	["Send Guild Friendly Database"] = true,
	
	["Show Chat Notifications"] = true,
    ["Toggles Showing Chat Notifications"] = true,
	
    ["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
	
	["Epeen Share"] = true,
	
	["Sent"] = true,
	["Notification"] = true,
	
	["Received"] = true,
	["from"] = true,
	
	["Scanning for duplicate entry in"] = true,
	["of"] = true,
	
	["discarded due to duplicate entry"] = true,
	
	["Adding"] = true,
	["to"] = true,
} end)

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---
EpeenShare = Epeen:NewModule("Share")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function EpeenShare:OnInitialize()
    self.db = Epeen:AcquireDBNamespace("Share")
	
    EpeenShare.defaultDB = {
        on = true,
		playerkossend = true,
		guildkossend = true,
		playerfriendlysend = true,
		guildfriendlysend = true,
		playerkosfilter = true,
		guildkosfilter = true,
		playerfriendlyfilter = true,
		guildfriendlyfilter = true,
		chatnotify = false,
    }
	
	Epeen:RegisterDefaults("Share", "profile", opts or EpeenShare.defaultDB or {})
	
    Epeen:ToggleModuleActive("Share", self.db.profile.on)
  
	
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
	Epeen.Options.args.settings.args.Share = {
        name = L["Share"],
        desc = L["Toggle sharing of Epeen Information"],
        type = "group",
		order = 105,
        args = { },
	}
	
	Epeen.Options.args.settings.args.Share.args.ShareHeader = {
		name = bluecolor..L["Share"].."  |c88888888"..Revision,
		icon = "Interface\\Addons\\Epeen\\icon.tga",
		iconHeight = 14,
		iconWidth = 14,
		order = 1,
		type = "header",
    }
	
	Epeen.Options.args.settings.args.Share.args.ShareHeaderBlank = {
		order = 2,
		type = "header",
	}
	
	Epeen.Options.args.settings.args.Share.args.ShareSharedDatabase = {
        name = L["Shared Database"],
        desc = L["Database to share automatically"],
		order = 100,
        type = "group",
        args = {
        	playerkos = {
            	name = string.format(L["Player KoS"], 2),
                desc = string.format(L["Toggles sending Player KoS broadcasts"], 2),
				order = 1,
                type = "toggle",
                get = function() return self.db.profile.playerkossend end,
                set = function() self.db.profile.playerkossend = not(self.db.profile.playerkossend) end
            },
			guildkos = {
                name = string.format(L["Guild KoS"], 1),
                desc = string.format(L["Toggles sending Guild KoS broadcasts"], 1),
				order = 2,
                type = "toggle",
                get = function() return self.db.profile.guildkossend end,
                set = function() self.db.profile.guildkossend = not(self.db.profile.guildkossend) end
            },
			playerfriendly = {
            	name = string.format(L["Player Friendly"], 2),
                desc = string.format(L["Toggles sending Player Friendly broadcasts"], 2),
				order = 3,
                type = "toggle",
                get = function() return self.db.profile.playerfriendlysend end,
                set = function() self.db.profile.playerfriendlysend = not(self.db.profile.playerfriendlysend) end
            },
			guildfriendly = {
                name = string.format(L["Guild Friendly"], 1),
                desc = string.format(L["Toggles sending Guild Friendly broadcasts"], 1),
				order = 4,
                type = "toggle",
                get = function() return self.db.profile.guildfriendlysend end,
                set = function() self.db.profile.guildfriendlysend = not(self.db.profile.guildfriendlysend) end
            },
		},
	}
	
	Epeen.Options.args.settings.args.Share.args.ShareReceiveFilter = {
        name = L["Receive Filter"],
        desc = L["Only accept broadcast sends for these databases"],
		order = 101,
        type = "group",
        args = {
        	playerkos = {
            	name = string.format(L["Player KoS"], 2),
                desc = string.format(L["Toggles receiving Player KoS broadcasts"], 2),
				order = 1,
                type = "toggle",
                get = function() return self.db.profile.playerkosfilter end,
                set = function() self.db.profile.playerkosfilter = not(self.db.profile.playerkosfilter) end
            },
			guildkos = {
                name = string.format(L["Guild KoS"], 1),
                desc = string.format(L["Toggles receiving Guild KoS broadcasts"], 1),
				order = 2,
                type = "toggle",
                get = function() return self.db.profile.guildkosfilter end,
                set = function() self.db.profile.guildkosfilter = not(self.db.profile.guildkosfilter) end
            },
			playerfriendly = {
            	name = string.format(L["Player Friendly"], 2),
                desc = string.format(L["Toggles receiving Player Friendly broadcasts"], 2),
				order = 3,
                type = "toggle",
                get = function() return self.db.profile.playerfriendlyfilter end,
                set = function() self.db.profile.playerfriendlyfilter = not(self.db.profile.playerfriendlyfilter) end
            },
			guildfriendly = {
                name = string.format(L["Guild Friendly"], 1),
                desc = string.format(L["Toggles receiving Guild Friendly broadcasts"], 1),
				order = 4,
                type = "toggle",
                get = function() return self.db.profile.guildfriendlyfilter end,
                set = function() self.db.profile.guildfriendlyfilter = not(self.db.profile.guildfriendlyfilter) end
            },
		},
	}
	
	Epeen.Options.args.settings.args.Share.args.ShareBroadcastDatabaseBlank = {
       	type = "header",
		order = 145,
	}
	
	Epeen.Options.args.settings.args.Share.args.ShareBroadcastDatabase = {
        name = L["Broadcast Database"],
        desc = L["Broadcast your entire database on your selected channel"],
		order = 150,
        type = "group",
        args = {
        	playerkos = {
            	name = string.format(L["Player KoS"], 2),
                desc = string.format(L["Send Player KoS Database"], 2),
				order = 1,
                type = "execute",
                func = function() self:SharePlayerKoS() end
            },
			guildkos = {
                name = string.format(L["Guild KoS"], 1),
                desc = string.format(L["Send Guild KoS Database"], 1),
				order = 2,
                type = "execute",
                func = function() self:ShareGuildKoS() end
            },
			playerfriendly = {
            	name = string.format(L["Player Friendly"], 2),
                desc = string.format(L["Send Player Friendly Database"], 2),
				order = 3,
                type = "execute",
                func = function() self:SharePlayerFriendly() end
            },
			guildfriendly = {
                name = string.format(L["Guild Friendly"], 1),
                desc = string.format(L["Send Guild Friendly Database"], 1),
				order = 4,
                type = "execute",
                func = function() self:ShareGuildFriendly() end
            },
		},
	}
	
	Epeen.Options.args.settings.args.Share.args.ShareChatNotifyBlank = {
       	type = "header",
		order = 174,
	}
	
	Epeen.Options.args.settings.args.Share.args.ShareChatNotify = {
		name = string.format(L["Show Chat Notifications"], 1),
        desc = string.format(L["Toggles Showing Chat Notifications"], 1),
		order = 175,
        type = "toggle",
        get = function() return self.db.profile.chatnotify end,
        set = function() self.db.profile.chatnotify = not(self.db.profile.chatnotify) end
	}
	
	Epeen.Options.args.settings.args.Share.args.ShareToggleBlank = {
       	type = "header",
		order = 195,
	}
	
    Epeen.Options.args.settings.args.Share.args.ShareToggle = {
    	name = L["Toggle"],
        desc = L["Toggle the Module On and Off"],
		order = 200,
        type = "toggle",
        get = function() return self.db.profile.on end,
        set = function() self.db.profile.on = Epeen:ToggleModuleActive("Share") end
    }
	self.title   = L["Epeen Share"]
	
	self.commPrefix = "Epeen-Share"
	self:SetCommPrefix("Epeen-Share")
	
end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function EpeenShare:OnEnable()
	self:RegisterEvent("EpeenShareSend");
	
	self:RegisterComm(self.commPrefix, "WHISPER", "OnCommReceive")
	self:RegisterComm(self.commPrefix, "GUILD", "OnCommReceive")
end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function EpeenShare:ResetDB()
	copyTable(self.db.profile, EpeenShare.defaultDB)
end

--- ========================================================= ---
--- Epeen CapitalizeWords
--- ========================================================= ---

Epeen_CapitalizeWords = function ( phrase )
	local phrase = string.gsub(phrase, "(_)"," ") 
	local words = Epeen_Split(phrase, " ");
	local capitalizedPhrase = "";

	for i=1,words.n do 
		local v = words[i];
		if ( i ~= 1 ) then
			capitalizedPhrase = capitalizedPhrase.." ";
		end
		capitalizedPhrase = capitalizedPhrase..string.upper(string.sub(v,1,1))..string.sub(v,2);
	end

	return capitalizedPhrase;
end;

--- ========================================================= ---
--- Epeen Split
--- ========================================================= ---

Epeen_Split = function ( text, separator )
	local t = {};
	t.n = 0;
	for value in string.gmatch(text,"[^"..separator.."]+") do
		t.n = t.n + 1;
		t[t.n] = value;
	end
	return t;
end;

--- ========================================================= ---
--- Epeen Share Player KoS
--- ========================================================= ---
function EpeenShare:SharePlayerKoS()
	if not EpeenKoS then
		return;
	end
	
	for i,data in pairs(EpeenKoS.db.profile.kosplayer) do
		self:TriggerEvent("EpeenShareSend", "Player KoS", data.name, "Bulk")
	end
	
end

--- ========================================================= ---
--- Epeen Share Guild KoS
--- ========================================================= ---
function EpeenShare:ShareGuildKoS()
	if not EpeenKoS then
		return;
	end

	for i,data in pairs(EpeenKoS.db.profile.kosguild) do
		self:TriggerEvent("EpeenShareSend", "Guild KoS", data.name, "Bulk")
	end

end

--- ========================================================= ---
--- Epeen Share Player Friendly
--- ========================================================= ---
function EpeenShare:SharePlayerFriendly()
	if not EpeenFriendly then
		return;
	end

	for i,data in pairs(EpeenFriendly.db.profile.friendlyplayer) do
		self:TriggerEvent("EpeenShareSend", "Player Friendly", data.name, "Bulk")
	end

end

--- ========================================================= ---
--- Epeen Share Guild Friendly
--- ========================================================= ---
function EpeenShare:ShareGuildFriendly()
	if not EpeenFriendly then
		return;
	end

	for i,data in pairs(EpeenFriendly.db.profile.friendlyplayer) do
		self:TriggerEvent("EpeenShareSend", "Player Friendly", data.name, "Bulk")
	end

end

--- ========================================================= ---
--- Epeen Share Send
--- ========================================================= ---
function EpeenShare:EpeenShareSend(ShareType, arg1, arg2)

	if ShareType == nil then
		return;
	end
	if arg1 == nil then
		return;
	end
	if arg2 == nil then
		return;
	end

	if ShareType == "Player KoS" then
		if arg2 == "Single" and not(self.db.profile.playerkossend) then
			return;
		end
	elseif ShareType == "Guild KoS" then
		if arg2 == "Single" and not(self.db.profile.guildkossend) then
			return;
		end
	elseif ShareType == "Player Friendly" then
		if arg2 == "Single" and not(self.db.profile.playerfriendlysend) then
			return;
		end
	elseif ShareType == "Guild Friendly" then
		if arg2 == "Single" and not(self.db.profile.guildfriendlysend) then
			return;
		end
	else
		return;
	end

	local ShareInfo = { }
	local ShareIsLastSeen = false;
	
	local ShareName = arg1;
	local ShareFaction = nil;
	local ShareLevel = nil;
	local ShareRace = nil;
	local ShareClass = nil;
	local ShareGuild = nil;
	local ShareGuildRank = nil;
	local ShareLastSeen = nil;

	if EpeenLastSeen then
		if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)] then
			ShareIsLastSeen = true;
			
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].faction then
				ShareFaction = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].faction;
			end
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].level then
				ShareLevel = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].level;
			end
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].race then
				ShareRace = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].race;
			end
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].class then
				ShareClass = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].class;
			end
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].guild then
				ShareGuild = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].guild;
			end
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].guildrank then
				ShareGuildRank = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].guildrank;
			end
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].lastseen then
				ShareLastSeen = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].lastseen;
			end
			if EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].server then
				ShareServer = EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)].server;
			else
				ShareServer = false;
			end
			
		end
	end
		
	if ShareIsLastSeen then
		local guildSuccess = self:SendCommMessage("GUILD", ShareType, ShareName, ShareServer, ShareFaction, ShareLevel, ShareRace, ShareClass, ShareLastSeen, ShareGuild, ShareGuildRank)
		local whisperSuccess = self:SendCommMessage("WHISPER", UnitName("player"), ShareType, ShareName, ShareServer, ShareFaction, ShareLevel, ShareRace, ShareClass, ShareLastSeen, ShareGuild, ShareGuildRank)
		return (guildSuccess and whisperSuccess)
	else
		local guildSuccess = self:SendCommMessage("GUILD", ShareType, ShareName)
		local whisperSuccess = self:SendCommMessage("WHISPER", UnitName("player"), ShareType, ShareName)
		return (guildSuccess and whisperSuccess)
	end

end


--- ========================================================= ---
--- On Comm Receive
--- ========================================================= ---

function EpeenShare:OnCommReceive(prefix, sender, distribution, ShareType, ShareName, ShareServer, ShareFaction, ShareLevel, ShareRace, ShareClass, ShareLastSeen, ShareGuild, ShareGuildRank)
	
	if not ShareType then
		return;
	end
	
	local EpeenDB = nil;
	local EpeenMN = nil;
	
	if ShareType == "Player KoS" then
		if not(self.db.profile.playerkosfilter) and sender ~= UnitName("player") then
			return;
		else
			EpeenDB = "kosplayer";
			EpeenMN = "EpeenKoS";
		end
	elseif ShareType == "Guild KoS" then
		if not(self.db.profile.guildkosfilter) and sender ~= UnitName("player") then
			return;
		else
			EpeenDB = "kosguild";
			EpeenMN = "EpeenKoS";
		end
	elseif ShareType == "Player Friendly" then
		if not(self.db.profile.playerfriendlyfilter) and sender ~= UnitName("player") then
			return;
		else
			EpeenDB = "friendlyplayer";
			EpeenMN = "EpeenFriendly";
		end
	elseif ShareType == "Guild Friendly" then
		if not(self.db.profile.guildfriendlyfilter) and sender ~= UnitName("player") then
			return;
		else
			EpeenDB = "friendlyguild";
			EpeenMN = "EpeenFriendly";
		end
	else
		return;
	end
	
	local ShareNamelc = string.gsub(string.lower(ShareName), "%s", "_");
	
	if sender ~= UnitName("player") then
		if self.db.profile.chatnotify then
			self:Print(L["Received"].." "..ShareType.." "..L["from"].." "..sender)
			self:Print(L["Scanning for duplicate entry in"].." "..ShareType.." "..L["of"].." "..Epeen_CapitalizeWords(ShareName))
		end
		
		if EpeenMN == "EpeenKoS" and EpeenKoS then
			if EpeenKoS.db.profile[EpeenDB][ShareNamelc] then
				if self.db.profile.chatnotify then
					self:Print(ShareType.." "..Epeen_CapitalizeWords(ShareName).." "..L["discarded due to duplicate entry"])
				end
			else
				if self.db.profile.chatnotify then
					self:Print(L["Adding"].." "..Epeen_CapitalizeWords(ShareName).." "..L["to"].." "..ShareType)
				end
				
				if ShareType == "Player KoS" then
					EpeenKoS.db.profile.kosplayer[ShareName] = {};
					EpeenKoS.db.profile.kosplayer[ShareName]["name"] = ShareName;
					EpeenKoS.db.profile.kosplayer[ShareName]["reason"] = "Added by "..sender;
					
					if ShareFaction ~= nil and not(EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)]) then
						if ShareServer == false then
							ShareServer = nil;
						end
						self:TriggerEvent("EpeenSeen", ShareFaction, Epeen_CapitalizeWords(ShareName), ShareServer, ShareLevel, ShareRace, ShareClass, ShareGuild, ShareGuildRank)
					end
					 
				elseif ShareType == "Guild KoS" then
					EpeenKoS.db.profile.kosguild[ShareName] = {};
					EpeenKoS.db.profile.kosguild[ShareName]["name"] = ShareName;
					EpeenKoS.db.profile.kosguild[ShareName]["reason"] = "Added by "..sender;
				end
				
			end
		end
		
		if EpeenMN == "EpeenFriendly" and EpeenFriendly then
			if EpeenFriendly.db.profile[EpeenDB][ShareNamelc] then
				if self.db.profile.chatnotify then
					self:Print(ShareType.." "..Epeen_CapitalizeWords(ShareName).." "..L["discarded due to duplicate entry"])
				end
			else
				if self.db.profile.chatnotify then
					self:Print(L["Adding"].." "..Epeen_CapitalizeWords(ShareName).." "..L["to"].." "..ShareType)
				end
				
				if ShareType == "Player Friendly" then
					EpeenFriendly.db.profile.friendlyplayer[ShareName] = {};
					EpeenFriendly.db.profile.friendlyplayer[ShareName]["name"] = ShareName;
					EpeenFriendly.db.profile.friendlyplayer[ShareName]["reason"] = "Added by "..sender;
					
					if ShareFaction ~= nil and not(EpeenLastSeen.db.profile.chars[Epeen_CapitalizeWords(ShareName)]) then
						if ShareServer == false then
							ShareServer = nil;
						end
						self:TriggerEvent("EpeenSeen", ShareFaction, Epeen_CapitalizeWords(ShareName), ShareServer, ShareLevel, ShareRace, ShareClass, ShareGuild, ShareGuildRank)
					end
					
				elseif ShareType == "Guild Friendly" then
					EpeenFriendly.db.profile.friendlyguild[ShareName] = {};
					EpeenFriendly.db.profile.friendlyguild[ShareName]["name"] = ShareName;
					EpeenFriendly.db.profile.friendlyguild[ShareName]["reason"] = "Added by "..sender;
				end
				
			end
		end
		
	else
		if self.db.profile.chatnotify then
			self:Print(L["Sent"].." "..ShareType.." "..L["Notification"]..": "..Epeen_CapitalizeWords(ShareName))
		end
	end
	
end