﻿--[[

Name: Epeen Stats
Revision: $Revision: 23844 $
Date: $Date: 2007-01-07 03:01:22 -0500 (Sun, 07 Jan 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: A module to show Epeen Stats.
Dependencies: Ace2, Epeen

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 23844 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("Stats")

L:RegisterTranslations("enUS", function() return {
    ["Stats"] = true,
    ["Toggle logging and displaying of Stats"] = true,
	
	["Epeen Stats"] = true,
	
	["Stats Settings"] = true,
	["Apperance"] = true,
    ["Change the Apperance of the Stats Frame"] = true,
	["Backdrop Color"] = true,
    ["Change the Stats Frame Backdrop Color"] = true,
	["Border Color"] = true,
    ["Change the Stats Frame Border Color"] = true,
	["Text Color"] = true,
    ["Change the Stats Frame Text Color"] = true,
	["Stats Frame Scale"] = true,
    ["Set the Stats Frame Scale"] = true,
	["Stats Frame Lock"] = true,
    ["Toggle the Stats Frame Lock Status"] = true,
	
	["Show Chat Notifications"] = true,
    ["Toggles Showing Chat Notifications"] = true,
	
    ["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
	
	["Show Stats Frame"] = true,
    ["Toggle the Stats Frame"] = true,
	
	["General Stats"] = true,
	["Total Kills"] = true,
	["Unique Kills"] = true,
	["Average Kill Level"] = true,
	["Total Deaths"] = true,
	["Unique Deaths"] = true,
	["Average Death Level"] = true,
	["Survivability"] = true,
	
	["Top 10 Killed Players"] = true,
	["Top 10 Deaths by Players"] = true,
	["Top 10 Killed Guilds"] = true,
	["Top 10 Deaths by Guilds"] = true,
	["Top 10 Killed Class"] = true,
	["Top 10 Deaths by Class"] = true,
	["Top 10 Killed Races"] = true,
	["Top 10 Deaths by Races"] = true,
	
} end)

Epeen_DateDropdownList = {
	{ id = 1, name = "Today", sortType = "today" },
	{ id = 2, name = "Total", sortType = "total" },
};

Epeen_TypeDropdownList = {
	{ id = 1, name = "General Stats", sortType = "general" },
	{ id = 2, name = "Top 10 Killed Players", sortType = "toptenkillplayers" },
	{ id = 3, name = "Top 10 Killed Guilds", sortType = "toptenkillguilds" },
	{ id = 4, name = "Top 10 Killed Class", sortType = "toptenkillclass" },
	{ id = 5, name = "Top 10 Killed Races", sortType = "toptenkillraces" },
	{ id = 6, name = "Top 10 Deaths by Players", sortType = "toptendeathplayers" },
	{ id = 7, name = "Top 10 Deaths by Guilds", sortType = "toptendeathguilds" },
	{ id = 8, name = "Top 10 Deaths by Class", sortType = "toptendeathclass" },
	{ id = 9, name = "Top 10 Deaths by Races", sortType = "toptendeathraces" },
};

--- ========================================================= ---
--- Add ParserLib
--- ========================================================= ---

local parser = ParserLib:GetInstance("1.1")

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---

EpeenStats = Epeen:NewModule("Stats")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function EpeenStats:OnInitialize()
    self.db = Epeen:AcquireDBNamespace("Stats")
	
    EpeenStats.defaultDB = {
        on = true,
		chatnotify = false,
		chars = { },
		currentdatesort = {1, "today"},
		currenttypesort = {1, "general"},
		lock = false,
		statsscale = 1,
		
		backdropcolor = {
            r = 0.10,
            g = 0.10,
            b = 0.10,
            a = 1.00,
        },
		bordercolor = {
            r = 0.90,
            g = 0.90,
            b = 0.90,
            a = 1.00,
        },
		textcolor = {
            r = 0.90,
            g = 0.90,
            b = 0.90,
            a = 1.00,
        },
    }
	
	Epeen:RegisterDefaults("Stats", "profile", opts or EpeenStats.defaultDB or {})
	
    Epeen:ToggleModuleActive("Stats", self.db.profile.on)
  
	
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
	Epeen.Options.args.settings.args.Stats = {
        name = L["Stats"],
        desc = L["Toggle logging and displaying of Stats"],
        type = "group",
		order = 105,
        args = { },
	}
	
	Epeen.Options.args.settings.args.Stats.args.StatsHeader = {
		name = bluecolor..L["Stats"].."  |c88888888"..Revision,
		icon = "Interface\\Addons\\Epeen\\icon.tga",
		iconHeight = 14,
		iconWidth = 14,
		order = 1,
		type = "header",
    }
	
	Epeen.Options.args.settings.args.Stats.args.ShareHeaderBlank = {
		order = 2,
		type = "header",
	}
	
	Epeen.Options.args.settings.args.Stats.args.StatsApperance = {
		name = L["Apperance"],
		desc = L["Change the Apperance of the Stats Frame"],
       	type = "group",
		order = 100,
		args = {
			header = {
				type = "header",
				name = bluecolor..L["Stats Settings"].."  |c88888888"..Revision,
				icon = "Interface\\Addons\\Epeen\\icon.tga",
				iconHeight = 14,
				iconWidth = 14,
				order = 1
            },
			blank = {
       			type = "header",
				order = 2,
			},
			backdropcolor = {
				name = L["Backdrop Color"],
				desc = L["Change the Stats Frame Backdrop Color"],
				order = 100,
    			type = 'color',
    			get = function() return 
					self.db.profile.backdropcolor.r,
                	self.db.profile.backdropcolor.g,
                	self.db.profile.backdropcolor.b,
					self.db.profile.backdropcolor.a
    			end,
    			set = function(r, g, b, a)
                	self.db.profile.backdropcolor.r,
                	self.db.profile.backdropcolor.g,
                	self.db.profile.backdropcolor.b,
					self.db.profile.backdropcolor.a
					= r, g, b, a;
					EpeenStats:SetBackdropColor();
    			end,
    			hasAlpha = true
			},
			bordercolor = {
				name = L["Border Color"],
				desc = L["Change the Stats Frame Border Color"],
				order = 200,
    			type = 'color',
    			get = function() return 
					self.db.profile.bordercolor.r,
                	self.db.profile.bordercolor.g,
                	self.db.profile.bordercolor.b,
					self.db.profile.bordercolor.a
    			end,
    			set = function(r, g, b, a)
                	self.db.profile.bordercolor.r,
                	self.db.profile.bordercolor.g,
                	self.db.profile.bordercolor.b,
					self.db.profile.bordercolor.a
					= r, g, b, a;
					EpeenStats:SetBorderColor();
    			end,
    			hasAlpha = true
			},
			textcolor = {
				name = L["Text Color"],
				desc = L["Change the Stats Frame Text Color"],
				order = 300,
    			type = 'color',
    			get = function() return 
					self.db.profile.textcolor.r,
                	self.db.profile.textcolor.g,
                	self.db.profile.textcolor.b,
					self.db.profile.textcolor.a
    			end,
    			set = function(r, g, b, a)
                	self.db.profile.textcolor.r,
                	self.db.profile.textcolor.g,
                	self.db.profile.textcolor.b,
					self.db.profile.textcolor.a
					= r, g, b, a;
					EpeenStats:SetTextColor();
    			end,
    			hasAlpha = true
			},
		}
	}
		
	Epeen.Options.args.settings.args.Stats.args.StatsScale = {
		name = L["Stats Frame Scale"],
		desc = L["Set the Stats Frame Scale"],
		order = 101,
		type = 'range',
		min  = 0.1,
		max  = 1.0,
		step = 0.05,
		set  = function(scale) self.db.profile.statsscale = scale;  EpeenStats:SetScale(); end,
		get  = function() return self.db.profile.statsscale end,
	}
	
	Epeen.Options.args.settings.args.Stats.args.StatsLock = {
        name = L["Stats Frame Lock"],
        desc = L["Toggle the Stats Frame Lock Status"],
		order = 102,
        type = "toggle",
        get = function() return self.db.profile.lock end,
        set = function() self.db.profile.lock = not self.db.profile.lock end
	}
	
	Epeen.Options.args.settings.args.Stats.args.StatsChatNotifyBlank = {
       	type = "header",
		order = 174,
	}
	
	Epeen.Options.args.settings.args.Stats.args.StatsChatNotify = {
		name = string.format(L["Show Chat Notifications"], 1),
        desc = string.format(L["Toggles Showing Chat Notifications"], 1),
		order = 175,
        type = "toggle",
        get = function() return self.db.profile.chatnotify end,
        set = function() self.db.profile.chatnotify = not(self.db.profile.chatnotify) end
	}
	
	Epeen.Options.args.settings.args.Stats.args.StatsToggleBlank = {
       	type = "header",
		order = 195,
	}
	
    Epeen.Options.args.settings.args.Stats.args.StatsToggle = {
    	name = L["Toggle"],
        desc = L["Toggle the Module On and Off"],
		order = 200,
        type = "toggle",
        get = function() return self.db.profile.on end,
        set = function() self.db.profile.on = Epeen:ToggleModuleActive("Stats") end
    }
	
	Epeen.Options.args.StatsFrameToggle = {
        name = L["Show Stats Frame"],
        desc = L["Toggle the Stats Frame"],
		order = 11,
        type = "execute",
		func = function() self:ToggleStatsFrame() end,
	}
	
	self.title   = L["Epeen Stats"]
	
	local lastDamageDealt = nil
	local lastDamageRecieved = nil
	
	local lossThrottle = nil
	local winsThrottle = nil
	
	Epeen_TimeOffSet = 1112600000;
	
end

--- ========================================================= ---
--- Utility
--- ========================================================= ---

local new, del
do
	local cache = setmetatable({},{__mode='k'})
	function new()
		local t = next(cache)
		if t then
			cache[t] = nil
			return t
		else
			return {}
		end
	end
	function del(t)
		if type(t) == "table" then
			for k in pairs(t) do
				t[k] = nil
			end
			cache[t] = true
		end
		return new()
	end
end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function EpeenStats:OnEnable()
	playerName = UnitName("player");
	
	self:RegisterEvent("PLAYER_DEAD")

	parser:RegisterEvent("EpeenStats", "CHAT_MSG_COMBAT_FRIENDLY_DEATH", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_COMBAT_HONOR_GAIN", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_COMBAT_HOSTILE_DEATH", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_COMBAT_PET_HITS", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_COMBAT_SELF_HITS", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_DAMAGESHIELDS_ON_SELF", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_PET_DAMAGE", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE", function(event, info) self:EventHandler(event, info) end)
	parser:RegisterEvent("EpeenStats", "CHAT_MSG_SPELL_SELF_DAMAGE", function(event, info) self:EventHandler(event, info) end)
	
	EpeenStatsFrame:SetBackdropColor(self.db.profile.backdropcolor.r, self.db.profile.backdropcolor.g, self.db.profile.backdropcolor.b, self.db.profile.backdropcolor.a);
	EpeenStatsFrame:SetBackdropBorderColor(self.db.profile.bordercolor.r, self.db.profile.bordercolor.g, self.db.profile.bordercolor.b, self.db.profile.bordercolor.a);
	EpeenStatsFrame_TitleText:SetTextColor(self.db.profile.textcolor.r, self.db.profile.textcolor.g, self.db.profile.textcolor.b, self.db.profile.textcolor.a);
	EpeenStatsFrame:SetScale(self.db.profile.statsscale);
	
	lossThrottle = del(lossThrottle)
	winsThrottle = del(winsThrottle)

end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function EpeenStats:ResetDB()
	copyTable(self.db.profile, EpeenStats.defaultDB)
end

--- ========================================================= ---
--- Toggle Last Seen Frame
--- ========================================================= ---

function EpeenStats:ToggleStatsFrame()
	if (EpeenStatsFrame:IsVisible()) then
		EpeenStatsFrame:Hide();
	else
		EpeenStatsFrame:Show();
		
		EpeenStatsFrame_TitleText:SetText("Epeen Stats v"..Revision);
		--EpeenStatsFrame_JustifyText
	end
end

--- ========================================================= ---
--- Check Stats Lock Status
--- ========================================================= ---

function EpeenStats:CheckLock()
	if not self.db.profile.lock then
		if(arg1 == "LeftButton") then
			EpeenStatsFrame:StartMoving();
		end
	end
end

--- ========================================================= ---
--- Set Stats Frame BackdropColor
--- ========================================================= ---

function EpeenStats:SetBackdropColor()
	EpeenStatsFrame:SetBackdropColor(self.db.profile.backdropcolor.r, self.db.profile.backdropcolor.g, self.db.profile.backdropcolor.b, self.db.profile.backdropcolor.a);
end

--- ========================================================= ---
--- Set Stats Frame BorderColor
--- ========================================================= ---

function EpeenStats:SetBorderColor()
	EpeenStatsFrame:SetBackdropBorderColor(self.db.profile.bordercolor.r, self.db.profile.bordercolor.g, self.db.profile.bordercolor.b, self.db.profile.bordercolor.a);
end

--- ========================================================= ---
--- Set Stats Frame TextColor
--- ========================================================= ---

function EpeenStats:SetTextColor()
	EpeenStatsFrame_TitleText:SetTextColor(self.db.profile.textcolor.r, self.db.profile.textcolor.g, self.db.profile.textcolor.b, self.db.profile.textcolor.a);
end

--- ========================================================= ---
--- Set Stats Frame Scale
--- ========================================================= ---

function EpeenStats:SetScale()
	EpeenStatsFrame:SetScale(self.db.profile.statsscale);
end

--- ========================================================= ---
--- Combat Messages
--- ========================================================= ---

function EpeenStats:EventHandler(event, info)

	if (self.db.profile.on == true) then

		if info.type == "hit" and info.source then
			if info.source == ParserLib_SELF then
				lastDamageDealt = info.victim
			elseif info.victim == ParserLib_SELF then
				lastDamageRecieved = info.source
			end
		elseif info.type == "death" then
			if info.victim == playerName then
				EpeenStats:PLAYER_DEAD(info.source)
			elseif info.victim == lastDamageDealt then
				EpeenStats:RecordWin(info.victim)
				lastDamageDealt = nil
			end
		elseif info.type == "honor" and info.source then
			--EpeenStats:RecordWin(info.source)
		end

	end
end

--- ========================================================= ---
--- Player Died
--- ========================================================= ---

function EpeenStats:PLAYER_DEAD(source)
	if not lastDamageRecieved or lastDamageRecieved == "" then
		if not source or source == "" then 
			return; 
		end
		lastDamageRecieved = source
	end

	EpeenStats:RecordLoss(lastDamageRecieved)
	lastDamageRecieved = nil
end

--- ========================================================= ---
--- Player Win
--- ========================================================= ---

function EpeenStats:RecordWin(source)
	if not source then 
		return; 
	end
	
	if winsThrottle[source] and ((winsThrottle[source] + 10) > GetTime()) then 
		return 
	end
	winsThrottle[source] = GetTime()

	if self.db.profile.chars[source] then
		self.db.profile.chars[source].win = self.db.profile.chars[source].win + 1;
	elseif EpeenLastSeen then
			if EpeenLastSeen.db.profile.chars[source] then
				self.db.profile.chars[source] = {};
				self.db.profile.chars[source]["name"] = source;
				self.db.profile.chars[source]["win"] = 1;
				self.db.profile.chars[source]["loss"] = 0;
			else
				return;
			end
	else
		return;
	end
	
	if self.db.profile.chatnotify then
		self:Print("Logging kill of "..source)
	end
	
	if (EpeenStatsFrame:IsVisible()) then
		EpeenStats:Refresh();
	end
end

--- ========================================================= ---
--- Player Loss
--- ========================================================= ---

function EpeenStats:RecordLoss(source)
	if not source then 
		return; 
	end

	if lossThrottle[source] and ((lossThrottle[source] + 10) > GetTime()) then 
		return 
	end
	lossThrottle[source] = GetTime()

	if self.db.profile.chars[source] then
		self.db.profile.chars[source].loss = self.db.profile.chars[source].loss + 1;
	elseif EpeenLastSeen then
			if EpeenLastSeen.db.profile.chars[source] then
				self.db.profile.chars[source] = {};
				self.db.profile.chars[source]["name"] = source;
				self.db.profile.chars[source]["win"] = 0;
				self.db.profile.chars[source]["loss"] = 1;
			else
				return;
			end
	else
		return;
	end
	
	if self.db.profile.chatnotify then
		self:Print("Logging death by "..source)
	end
	
	if (EpeenStatsFrame:IsVisible()) then
		EpeenStats:Refresh();
	end
end

--- ========================================================= ---
--- Epeen Item Button (On Click)
--- ========================================================= ---

function EpeenStats:ItemButton_OnClick(arg1)

end

--- ========================================================= ---
--- Epeen Item Button (On Enter)
--- ========================================================= ---

function EpeenStats:ItemButton_OnEnter()

end

--- ========================================================= ---
--- Epeen Item Button (On Leave)
--- ========================================================= ---

function EpeenStats:ItemButton_OnLeave()

end

--- ========================================================= ---
--- Epeen Date DropDown Initialize
--- ========================================================= ---

function EpeenStats:DateDropDown_Initialize()
	local info;
	for i = 1, getn(Epeen_DateDropdownList), 1 do
		info = { };
		info.text = Epeen_DateDropdownList[i].name;
		info.func = EpeenStats.DateDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

--- ========================================================= ---
--- Epeen Date DropDown OnLoad
--- ========================================================= ---

function EpeenStats:DateDropDown_OnLoad()
	UIDropDownMenu_Initialize(EpeenStatsFrame_DateDropDown, EpeenStats.DateDropDown_Initialize);
    EpeenStats:DateDropDown_SetSelectedID(EpeenStatsFrame_DateDropDown, 1, Epeen_DateDropdownList);
	UIDropDownMenu_SetWidth(60);
	UIDropDownMenu_SetButtonWidth(24);
	UIDropDownMenu_JustifyText("LEFT", EpeenStatsFrame_DateDropDown)
end

--- ========================================================= ---
--- Epeen Date DropDown OnShow
--- ========================================================= ---

function EpeenStats:DateDropDown_OnShow()
    EpeenStats:DateDropDown_SetSelectedID(EpeenStatsFrame_DateDropDown, EpeenStats.db.profile.currentdatesort[1], Epeen_DateDropdownList);
end

--- ========================================================= ---
--- Epeen Date DropDown OnClick
--- ========================================================= ---

function EpeenStats:DateDropDown_OnClick()
   local oldID = UIDropDownMenu_GetSelectedID(EpeenStatsFrame_DateDropDown);
	UIDropDownMenu_SetSelectedID(EpeenStatsFrame_DateDropDown, this:GetID());
	if( oldID ~= this:GetID() ) then
	   EpeenStats.db.profile.currentdatesort[1] = Epeen_DateDropdownList[this:GetID()].id
	   EpeenStats.db.profile.currentdatesort[2] = Epeen_DateDropdownList[this:GetID()].sortType;
   	   EpeenStats:Refresh();
	end
end

--- ========================================================= ---
--- Epeen Date DropDown SetSelected
--- ========================================================= ---

function EpeenStats:DateDropDown_SetSelectedID(frame, id, names)
	UIDropDownMenu_SetSelectedID(frame, id);
	if( not frame ) then
		frame = this;
	end
	UIDropDownMenu_SetText(names[id].name, frame);
end

--- ========================================================= ---
--- Epeen Type DropDown Initialize
--- ========================================================= ---

function EpeenStats:TypeDropDown_Initialize()
	local info;
	for i = 1, getn(Epeen_TypeDropdownList), 1 do
		info = { };
		info.text = Epeen_TypeDropdownList[i].name;
		info.func = EpeenStats.TypeDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

--- ========================================================= ---
--- Epeen Type DropDown OnLoad
--- ========================================================= ---

function EpeenStats:TypeDropDown_OnLoad()
	UIDropDownMenu_Initialize(EpeenStatsFrame_TypeDropDown, EpeenStats.TypeDropDown_Initialize);
    EpeenStats:TypeDropDown_SetSelectedID(EpeenStatsFrame_TypeDropDown, 1, Epeen_TypeDropdownList);
	UIDropDownMenu_SetWidth(80);
	UIDropDownMenu_SetButtonWidth(24);
	UIDropDownMenu_JustifyText("LEFT", EpeenStatsFrame_TypeDropDown)
end

--- ========================================================= ---
--- Epeen Type DropDown OnShow
--- ========================================================= ---

function EpeenStats:TypeDropDown_OnShow()
    EpeenStats:TypeDropDown_SetSelectedID(EpeenStatsFrame_TypeDropDown, EpeenStats.db.profile.currenttypesort[1], Epeen_TypeDropdownList);
end

--- ========================================================= ---
--- Epeen Type DropDown OnClick
--- ========================================================= ---

function EpeenStats:TypeDropDown_OnClick()
   local oldID = UIDropDownMenu_GetSelectedID(EpeenStatsFrame_TypeDropDown);
	UIDropDownMenu_SetSelectedID(EpeenStatsFrame_TypeDropDown, this:GetID());
	if( oldID ~= this:GetID() ) then
	   EpeenStats.db.profile.currenttypesort[1] = Epeen_TypeDropdownList[this:GetID()].id
	   EpeenStats.db.profile.currenttypesort[2] = Epeen_TypeDropdownList[this:GetID()].sortType;
   	   EpeenStats:Refresh();
	end
end

--- ========================================================= ---
--- Epeen Type DropDown SetSelected
--- ========================================================= ---

function EpeenStats:TypeDropDown_SetSelectedID(frame, id, names)
	UIDropDownMenu_SetSelectedID(frame, id);
	if( not frame ) then
		frame = this;
	end
	UIDropDownMenu_SetText(names[id].name, frame);
end

--- ========================================================= ---
--- Epeen Refresh
--- ========================================================= ---

function EpeenStats:Refresh()
	
	if EpeenStats.db.profile.currenttypesort[2] == "general" then
		
		local totalkills = 0;
		local uniquekills = 0;
		local totalkillslevel = 0;
		local totaldeaths = 0;
		local uniquedeaths = 0;
		local totaldeathslevel = 0;
		
		for i,data in pairs(EpeenStats.db.profile.chars) do
		
			if EpeenStats.db.profile.currentdatesort[2] == "total" then
				totalkills = totalkills + data.win
				totaldeaths = totaldeaths + data.loss
			
				if data.win > 0 then
					uniquekills = uniquekills + 1;
				end
			
				if data.loss > 0 then
					uniquedeaths = uniquedeaths + 1;
				end
			
				if EpeenLastSeen then
					if EpeenLastSeen.db.profile.chars[data.name] then
						if data.win > 0 then
							totalkillslevel = totalkillslevel + (data.win * EpeenLastSeen.db.profile.chars[data.name].level);
						end
						if data.loss > 0 then
							totaldeathslevel = totaldeathslevel + (data.loss * EpeenLastSeen.db.profile.chars[data.name].level);
						end
					end
				end
			
			elseif EpeenStats.db.profile.currentdatesort[2] == "today" then
				
				local strtodaydate = date("%m %d %y", time());
				
				if EpeenLastSeen then
					if EpeenLastSeen.db.profile.chars[data.name] then
						strtodaydate = date("%m %d %y", time());
						playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[data.name].lastseen + Epeen_TimeOffSet))
						if strtodaydate == playerseen then

							totalkills = totalkills + data.win
							totaldeaths = totaldeaths + data.loss
			
							if data.win > 0 then
								uniquekills = uniquekills + 1;
							end
			
							if data.loss > 0 then
								uniquedeaths = uniquedeaths + 1;
							end
			
							if EpeenLastSeen then
								if EpeenLastSeen.db.profile.chars[data.name] then
									if data.win > 0 then
										totalkillslevel = totalkillslevel + (data.win * EpeenLastSeen.db.profile.chars[data.name].level);
									end
									if data.loss > 0 then
										totaldeathslevel = totaldeathslevel + (data.loss * EpeenLastSeen.db.profile.chars[data.name].level);
									end
								end
							end
						end
					end
				end
			end
		end
		
		local avgkillslevel = 0;
		if totalkillslevel ~= 0 and totalkills ~= 0 then
			avgkillslevel = totalkillslevel / totalkills;
		end
		
		local avgdeathslevel = 0;
		if totaldeathslevel ~= 0 and totaldeaths ~= 0 then
			avgdeathslevel = totaldeathslevel / totaldeaths;
		end
		
		local totalfights = totalkills + totaldeaths;
		local survivability = 100;
		if( totalfights > 0 ) then
	  			survivability = (100/totalfights) * totalkills;
		end
		
		avgkillslevel = string.sub(avgkillslevel, 0, 4);
		avgdeathslevel = string.sub(avgdeathslevel, 0, 4);
		survivability = string.sub(survivability, 0, 4) .. '%';
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["General Stats"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		local Item3 = getglobal("EpeenStatsFrame_Item3");
		getglobal("EpeenStatsFrame_Item3Label"):SetText(L["Total Kills"]);
		getglobal("EpeenStatsFrame_Item3Info"):SetText(totalkills);
		
		local Item4 = getglobal("EpeenStatsFrame_Item4");
		getglobal("EpeenStatsFrame_Item4Label"):SetText(L["Unique Kills"]);
		getglobal("EpeenStatsFrame_Item4Info"):SetText(uniquekills);
		
		local Item5 = getglobal("EpeenStatsFrame_Item5");
		getglobal("EpeenStatsFrame_Item5Label"):SetText(L["Average Kill Level"]);
		getglobal("EpeenStatsFrame_Item5Info"):SetText(avgkillslevel);
		
		local Item6 = getglobal("EpeenStatsFrame_Item6");
		getglobal("EpeenStatsFrame_Item6Label"):SetText("");
		getglobal("EpeenStatsFrame_Item6Info"):SetText("");
		
		local Item7 = getglobal("EpeenStatsFrame_Item7");
		getglobal("EpeenStatsFrame_Item7Label"):SetText(L["Total Deaths"]);
		getglobal("EpeenStatsFrame_Item7Info"):SetText(totaldeaths);
		
		local Item8 = getglobal("EpeenStatsFrame_Item8");
		getglobal("EpeenStatsFrame_Item8Label"):SetText("Unique Deaths");
		getglobal("EpeenStatsFrame_Item8Info"):SetText(uniquedeaths);
		
		local Item9 = getglobal("EpeenStatsFrame_Item9");
		getglobal("EpeenStatsFrame_Item9Label"):SetText(L["Average Death Level"]);
		getglobal("EpeenStatsFrame_Item9Info"):SetText(avgdeathslevel);
		
		local Item10 = getglobal("EpeenStatsFrame_Item10");
		getglobal("EpeenStatsFrame_Item10Label"):SetText("");
		getglobal("EpeenStatsFrame_Item10Info"):SetText("");
		
		local Item11 = getglobal("EpeenStatsFrame_Item11");
		getglobal("EpeenStatsFrame_Item11Label"):SetText(L["Survivability"]);
		getglobal("EpeenStatsFrame_Item11Info"):SetText(survivability);
		
		local Item12 = getglobal("EpeenStatsFrame_Item12");
		getglobal("EpeenStatsFrame_Item12Label"):SetText("");
		getglobal("EpeenStatsFrame_Item12Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		Item3:Show();
		Item4:Show();
		Item5:Show();
		Item6:Show();
		Item6:Show();
		Item7:Show();
		Item8:Show();
		Item9:Show();
		Item10:Show();
		Item11:Show();
		Item12:Show();

	elseif EpeenStats.db.profile.currenttypesort[2] == "toptenkillplayers" then
		
		local iNew = 3;
		topkilledsorted = { }
		StatsFrame_Items = { };
		
		for k,v in pairs(EpeenStats.db.profile.chars) do
			if v.win > 0 then
				if EpeenStats.db.profile.currentdatesort[2] == "today" then
					strtodaydate = date("%m %d %y", time());
					playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[v.name].lastseen + Epeen_TimeOffSet))
					if strtodaydate == playerseen then
						table.insert(topkilledsorted, v)
					end
				else
					table.insert(topkilledsorted, v)
				end
			end
		end
		
		if getn(topkilledsorted) > 1 then
			table.sort(topkilledsorted, function (a, b) if a["win"] == b["win"] then return a["name"] < b["name"] else return a["win"] > b["win"] end end)
		end

		for i, data in pairs(topkilledsorted) do
    		StatsFrame_Items[iNew] = data.name;
			iNew = iNew + 1;
		end
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["Top 10 Killed Players"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		
		for iItem = 3, 12, 1 do
			local DBItem = getglobal("EpeenStatsFrame_Item"..iItem);
			local name = StatsFrame_Items[iItem];
			local currentItem = nil;
			
			if EpeenStats.db.profile.chars[name] then
				currentItem = EpeenStats.db.profile.chars[name];
			end
			
			if currentItem ~= nil then
				getglobal("EpeenStatsFrame_Item"..iItem.."Label"):SetText(currentItem.name);
				getglobal("EpeenStatsFrame_Item"..iItem.."Info"):SetText(currentItem.win);
			
				DBItem:Show();
			else
				DBItem:Hide();
			end
		end
	elseif EpeenStats.db.profile.currenttypesort[2] == "toptendeathplayers" then
		
		local iNew = 3;
		topdeathsorted = { }
		StatsFrame_Items = { };
		
		for k,v in pairs(EpeenStats.db.profile.chars) do
			if v.loss > 0 then
				if EpeenStats.db.profile.currentdatesort[2] == "today" then
					strtodaydate = date("%m %d %y", time());
					playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[v.name].lastseen + Epeen_TimeOffSet))
					if strtodaydate == playerseen then
						table.insert(topdeathsorted, v)
					end
				else
					table.insert(topdeathsorted, v)
				end
			end
		end
		
		if getn(topdeathsorted) > 1 then
			table.sort(topdeathsorted, function (a, b) if a["loss"] == b["loss"] then return a["name"] < b["name"] else return a["loss"] > b["loss"] end end)
		end

		for i, data in pairs(topdeathsorted) do
    		StatsFrame_Items[iNew] = data.name;
			iNew = iNew + 1;
		end
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["Top 10 Deaths by Players"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		
		for iItem = 3, 12, 1 do
			local DBItem = getglobal("EpeenStatsFrame_Item"..iItem);
			local name = StatsFrame_Items[iItem];
			local currentItem = nil;
			
			if EpeenStats.db.profile.chars[name] then
				currentItem = EpeenStats.db.profile.chars[name];
			end
			
			if currentItem ~= nil then
				getglobal("EpeenStatsFrame_Item"..iItem.."Label"):SetText(currentItem.name);
				getglobal("EpeenStatsFrame_Item"..iItem.."Info"):SetText(currentItem.loss);
			
				DBItem:Show();
			else
				DBItem:Hide();
			end
		end
	elseif EpeenStats.db.profile.currenttypesort[2] == "toptenkillguilds" then
		
		local iNew = 3;
		local iGuild = nil;
		topkilledsorted = { }
		topkilledguild = { }
		topkilledguildsorted = { }
		StatsFrame_Items = { };
		
		for k,v in pairs(EpeenStats.db.profile.chars) do
			if v.win > 0 then
				if EpeenStats.db.profile.currentdatesort[2] == "today" then
					strtodaydate = date("%m %d %y", time());
					playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[v.name].lastseen + Epeen_TimeOffSet))
					if strtodaydate == playerseen then
						table.insert(topkilledsorted, v)
					end
				else
					table.insert(topkilledsorted, v)
				end
			end
		end

		for i, data in pairs(topkilledsorted) do
			if EpeenLastSeen then
				if EpeenLastSeen.db.profile.chars[data.name] then
					if EpeenLastSeen.db.profile.chars[data.name].guild and EpeenLastSeen.db.profile.chars[data.name].guild ~= "" then
						iGuild = EpeenLastSeen.db.profile.chars[data.name].guild
					
						if topkilledguild[iGuild] == nil then
							topkilledguild[iGuild] = { };
							topkilledguild[iGuild].name = iGuild;
							topkilledguild[iGuild].win = 0;
						end
						topkilledguild[iGuild].win = topkilledguild[iGuild].win + EpeenStats.db.profile.chars[data.name].win
					end
				end
			end
		end
		
		for k,v in pairs(topkilledguild) do
			table.insert(topkilledguildsorted, v)
		end
		
		if getn(topkilledguildsorted) > 1 then
			table.sort(topkilledguildsorted, function (a, b) if a["win"] == b["win"] then return a["name"] < b["name"] else return a["win"] > b["win"] end end)
		end

		for i, data in pairs(topkilledguildsorted) do
    		StatsFrame_Items[iNew] = data.name;
			iNew = iNew + 1;
		end
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["Top 10 Killed Guilds"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		
		for iItem = 3, 12, 1 do
			local DBItem = getglobal("EpeenStatsFrame_Item"..iItem);
			local name = StatsFrame_Items[iItem];
			local currentItem = nil;
			
			if topkilledguild[name] then
				currentItem = topkilledguild[name];
			end
			
			if currentItem ~= nil then
				getglobal("EpeenStatsFrame_Item"..iItem.."Label"):SetText(currentItem.name);
				getglobal("EpeenStatsFrame_Item"..iItem.."Info"):SetText(currentItem.win);
			
				DBItem:Show();
			else
				DBItem:Hide();
			end
		end
		
	elseif EpeenStats.db.profile.currenttypesort[2] == "toptendeathguilds" then
		
		local iNew = 3;
		local iGuild = nil;
		topdeathsorted = { }
		topdeathguild = { }
		topdeathguildsorted = { }
		StatsFrame_Items = { };
		
		for k,v in pairs(EpeenStats.db.profile.chars) do
			if v.loss > 0 then
				if EpeenStats.db.profile.currentdatesort[2] == "today" then
					strtodaydate = date("%m %d %y", time());
					playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[v.name].lastseen + Epeen_TimeOffSet))
					if strtodaydate == playerseen then
						table.insert(topdeathsorted, v)
					end
				else
					table.insert(topdeathsorted, v)
				end
			end
		end

		for i, data in pairs(topdeathsorted) do
			if EpeenLastSeen then
				if EpeenLastSeen.db.profile.chars[data.name] then
					if EpeenLastSeen.db.profile.chars[data.name].guild and EpeenLastSeen.db.profile.chars[data.name].guild ~= "" then
						iGuild = EpeenLastSeen.db.profile.chars[data.name].guild
					
						if topdeathguild[iGuild] == nil then
							topdeathguild[iGuild] = { };
							topdeathguild[iGuild].name = iGuild;
							topdeathguild[iGuild].loss = 0;
						end
						topdeathguild[iGuild].loss = topdeathguild[iGuild].loss + EpeenStats.db.profile.chars[data.name].loss
					end
				end
			end
		end
		
		for k,v in pairs(topdeathguild) do
			table.insert(topdeathguildsorted, v)
		end
		
		if getn(topdeathguildsorted) > 1 then
			table.sort(topdeathguildsorted, function (a, b) if a["loss"] == b["loss"] then return a["name"] < b["name"] else return a["loss"] > b["loss"] end end)
		end

		for i, data in pairs(topdeathguildsorted) do
    		StatsFrame_Items[iNew] = data.name;
			iNew = iNew + 1;
		end
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["Top 10 Deaths by Guilds"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		
		for iItem = 3, 12, 1 do
			local DBItem = getglobal("EpeenStatsFrame_Item"..iItem);
			local name = StatsFrame_Items[iItem];
			local currentItem = nil;

			if topdeathguild[name] then
				currentItem = topdeathguild[name];
			end
			
			if currentItem ~= nil then
				getglobal("EpeenStatsFrame_Item"..iItem.."Label"):SetText(currentItem.name);
				getglobal("EpeenStatsFrame_Item"..iItem.."Info"):SetText(currentItem.loss);
			
				DBItem:Show();
			else
				DBItem:Hide();
			end
		end
	elseif EpeenStats.db.profile.currenttypesort[2] == "toptenkillclass" then
		
		local iNew = 3;
		local iClass = nil;
		topkilledsorted = { }
		topkilledclass = { }
		topkilledclasssorted = { }
		StatsFrame_Items = { };
		
		for k,v in pairs(EpeenStats.db.profile.chars) do
			if v.win > 0 then
				if EpeenStats.db.profile.currentdatesort[2] == "today" then
					strtodaydate = date("%m %d %y", time());
					playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[v.name].lastseen + Epeen_TimeOffSet))
					if strtodaydate == playerseen then
						table.insert(topkilledsorted, v)
					end
				else
					table.insert(topkilledsorted, v)
				end
			end
		end

		for i, data in pairs(topkilledsorted) do
			if EpeenLastSeen then
				if EpeenLastSeen.db.profile.chars[data.name] then
					if EpeenLastSeen.db.profile.chars[data.name].class and EpeenLastSeen.db.profile.chars[data.name].class ~= "" then
						iClass = EpeenLastSeen.db.profile.chars[data.name].class
					
						if topkilledclass[iClass] == nil then
							topkilledclass[iClass] = { };
							topkilledclass[iClass].name = iClass;
							topkilledclass[iClass].win = 0;
						end
						topkilledclass[iClass].win = topkilledclass[iClass].win + EpeenStats.db.profile.chars[data.name].win
					end
				end
			end
		end
		
		for k,v in pairs(topkilledclass) do
			table.insert(topkilledclasssorted, v)
		end
		
		if getn(topkilledclasssorted) > 1 then
			table.sort(topkilledclasssorted, function (a, b) if a["win"] == b["win"] then return a["name"] < b["name"] else return a["win"] > b["win"] end end)
		end

		for i, data in pairs(topkilledclasssorted) do
    		StatsFrame_Items[iNew] = data.name;
			iNew = iNew + 1;
		end
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["Top 10 Killed Class"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		
		for iItem = 3, 12, 1 do
			local DBItem = getglobal("EpeenStatsFrame_Item"..iItem);
			local name = StatsFrame_Items[iItem];
			local currentItem = nil;

			if topkilledclass[name] then
				currentItem = topkilledclass[name];
			end
			
			if currentItem ~= nil then
				getglobal("EpeenStatsFrame_Item"..iItem.."Label"):SetText(currentItem.name);
				getglobal("EpeenStatsFrame_Item"..iItem.."Info"):SetText(currentItem.win);
			
				DBItem:Show();
			else
				DBItem:Hide();
			end
		end
	elseif EpeenStats.db.profile.currenttypesort[2] == "toptendeathclass" then
		
		local iNew = 3;
		local iClass = nil;
		topdeathsorted = { }
		topdeathclass = { }
		topdeathclasssorted = { }
		StatsFrame_Items = { };
		
		for k,v in pairs(EpeenStats.db.profile.chars) do
			if v.loss > 0 then
				if EpeenStats.db.profile.currentdatesort[2] == "today" then
					strtodaydate = date("%m %d %y", time());
					playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[v.name].lastseen + Epeen_TimeOffSet))
					if strtodaydate == playerseen then
						table.insert(topdeathsorted, v)
					end
				else
					table.insert(topdeathsorted, v)
				end
			end
		end

		for i, data in pairs(topdeathsorted) do
			if EpeenLastSeen then
				if EpeenLastSeen.db.profile.chars[data.name] then
					if EpeenLastSeen.db.profile.chars[data.name].class and EpeenLastSeen.db.profile.chars[data.name].class ~= "" then
						iClass = EpeenLastSeen.db.profile.chars[data.name].class
					
						if topdeathclass[iClass] == nil then
							topdeathclass[iClass] = { };
							topdeathclass[iClass].name = iClass;
							topdeathclass[iClass].loss = 0;
						end
						topdeathclass[iClass].loss = topdeathclass[iClass].loss + EpeenStats.db.profile.chars[data.name].loss
					end
				end
			end
		end
		
		for k,v in pairs(topdeathclass) do
			table.insert(topdeathclasssorted, v)
		end
		
		if getn(topdeathclasssorted) > 1 then
			table.sort(topdeathclasssorted, function (a, b) if a["loss"] == b["loss"] then return a["name"] < b["name"] else return a["loss"] > b["loss"] end end)
		end

		for i, data in pairs(topdeathclasssorted) do
    		StatsFrame_Items[iNew] = data.name;
			iNew = iNew + 1;
		end
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["Top 10 Deaths by Class"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		
		for iItem = 3, 12, 1 do
			local DBItem = getglobal("EpeenStatsFrame_Item"..iItem);
			local name = StatsFrame_Items[iItem];
			local currentItem = nil;

			if topdeathclass[name] then
				currentItem = topdeathclass[name];
			end
			
			if currentItem ~= nil then
				getglobal("EpeenStatsFrame_Item"..iItem.."Label"):SetText(currentItem.name);
				getglobal("EpeenStatsFrame_Item"..iItem.."Info"):SetText(currentItem.loss);
			
				DBItem:Show();
			else
				DBItem:Hide();
			end
		end
	elseif EpeenStats.db.profile.currenttypesort[2] == "toptenkillraces" then
		
		local iNew = 3;
		local iRaces = nil;
		topkilledsorted = { }
		topkilledraces = { }
		topkilledracessorted = { }
		StatsFrame_Items = { };
		
		for k,v in pairs(EpeenStats.db.profile.chars) do
			if v.win > 0 then
				if EpeenStats.db.profile.currentdatesort[2] == "today" then
					strtodaydate = date("%m %d %y", time());
					playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[v.name].lastseen + Epeen_TimeOffSet))
					if strtodaydate == playerseen then
						table.insert(topkilledsorted, v)
					end
				else
					table.insert(topkilledsorted, v)
				end
			end
		end

		for i, data in pairs(topkilledsorted) do
			if EpeenLastSeen then
				if EpeenLastSeen.db.profile.chars[data.name] then
					if EpeenLastSeen.db.profile.chars[data.name].race and EpeenLastSeen.db.profile.chars[data.name].race ~= "" then
						iRaces = EpeenLastSeen.db.profile.chars[data.name].race
					
						if topkilledraces[iRaces] == nil then
							topkilledraces[iRaces] = { };
							topkilledraces[iRaces].name = iRaces;
							topkilledraces[iRaces].win = 0;
						end
						topkilledraces[iRaces].win = topkilledraces[iRaces].win + EpeenStats.db.profile.chars[data.name].win
					end
				end
			end
		end
		
		for k,v in pairs(topkilledraces) do
			table.insert(topkilledracessorted, v)
		end
		
		if getn(topkilledracessorted) > 1 then
			table.sort(topkilledracessorted, function (a, b) if a["win"] == b["win"] then return a["name"] < b["name"] else return a["win"] > b["win"] end end)
		end

		for i, data in pairs(topkilledracessorted) do
    		StatsFrame_Items[iNew] = data.name;
			iNew = iNew + 1;
		end
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["Top 10 Killed Races"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		
		for iItem = 3, 12, 1 do
			local DBItem = getglobal("EpeenStatsFrame_Item"..iItem);
			local name = StatsFrame_Items[iItem];
			local currentItem = nil;

			if topkilledraces[name] then
				currentItem = topkilledraces[name];
			end
			
			if currentItem ~= nil then
				getglobal("EpeenStatsFrame_Item"..iItem.."Label"):SetText(currentItem.name);
				getglobal("EpeenStatsFrame_Item"..iItem.."Info"):SetText(currentItem.win);
			
				DBItem:Show();
			else
				DBItem:Hide();
			end
		end
	elseif EpeenStats.db.profile.currenttypesort[2] == "toptendeathraces" then
		
		local iNew = 3;
		local iRaces = nil;
		topdeathsorted = { }
		topdeathraces = { }
		topdeathracessorted = { }
		StatsFrame_Items = { };
		
		for k,v in pairs(EpeenStats.db.profile.chars) do
			if v.loss > 0 then
				if EpeenStats.db.profile.currentdatesort[2] == "today" then
					strtodaydate = date("%m %d %y", time());
					playerseen = date("%m %d %y", (EpeenLastSeen.db.profile.chars[v.name].lastseen + Epeen_TimeOffSet))
					if strtodaydate == playerseen then
						table.insert(topdeathsorted, v)
					end
				else
					table.insert(topdeathsorted, v)
				end
			end
		end

		for i, data in pairs(topdeathsorted) do
			if EpeenLastSeen then
				if EpeenLastSeen.db.profile.chars[data.name] then
					if EpeenLastSeen.db.profile.chars[data.name].race and EpeenLastSeen.db.profile.chars[data.name].race ~= "" then
						iRaces = EpeenLastSeen.db.profile.chars[data.name].race
					
						if topdeathraces[iRaces] == nil then
							topdeathraces[iRaces] = { };
							topdeathraces[iRaces].name = iRaces;
							topdeathraces[iRaces].loss = 0;
						end
						topdeathraces[iRaces].loss = topdeathraces[iRaces].loss + EpeenStats.db.profile.chars[data.name].loss
					end
				end
			end
		end
		
		for k,v in pairs(topdeathraces) do
			table.insert(topdeathracessorted, v)
		end
		
		if getn(topdeathracessorted) > 1 then
			table.sort(topdeathracessorted, function (a, b) if a["loss"] == b["loss"] then return a["name"] < b["name"] else return a["loss"] > b["loss"] end end)
		end

		for i, data in pairs(topdeathracessorted) do
    		StatsFrame_Items[iNew] = data.name;
			iNew = iNew + 1;
		end
		
		local Item1 = getglobal("EpeenStatsFrame_Item1");
		getglobal("EpeenStatsFrame_Item1Label"):SetText(L["Top 10 Deaths by Races"]);
		getglobal("EpeenStatsFrame_Item1Info"):SetText("");
		
		local Item2 = getglobal("EpeenStatsFrame_Item2");
		getglobal("EpeenStatsFrame_Item2Label"):SetText("-------------------");
		getglobal("EpeenStatsFrame_Item2Info"):SetText("");
		
		Item1:Show();
		Item2:Show();
		
		for iItem = 3, 12, 1 do
			local DBItem = getglobal("EpeenStatsFrame_Item"..iItem);
			local name = StatsFrame_Items[iItem];
			local currentItem = nil;

			if topdeathraces[name] then
				currentItem = topdeathraces[name];
			end
			
			if currentItem ~= nil then
				getglobal("EpeenStatsFrame_Item"..iItem.."Label"):SetText(currentItem.name);
				getglobal("EpeenStatsFrame_Item"..iItem.."Info"):SetText(currentItem.loss);
			
				DBItem:Show();
			else
				DBItem:Hide();
			end
		end
	end
end