﻿--[[

Name: Epeen Tooltip
Revision: $Revision: 27605 $
Date: $Date: 2007-02-09 01:32:41 -0500 (Fri, 09 Feb 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Epeen
Documentation: http://www.wowace.com/wiki/Epeen
SVN: http://svn.wowace.com/root/trunk/Epeen
Description: A module to show Epeen information in tooltip.
Dependencies: Ace2, Epeen

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 27605 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("Tooltip")

L:RegisterTranslations("enUS", function() return {
    ["Tooltip"] = true,
    ["Toggle showing Epeen information in the tooltip"] = true,
	["Alert"] = true,
	["Toggle whether or not to alert on Tooltips"] = true,		
	["Play Alert Sound"] = true,
	["Toggle whether or not to play sound on alert"] = true,
	["Alert Cooldown"] = true,
	["Set the Alert Cooldown in minutes"] = true,
	["Alert Cooldown"] = true,
	["Set the Alert Cooldown in minutes"] = true,
    ["Track"] = true,
    ["Toggle whether or not to track on Tooltips"] = true,
	["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
} end)

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---

EpeenTooltip = Epeen:NewModule("Tooltip")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function EpeenTooltip:OnInitialize()
    self.db = Epeen:AcquireDBNamespace("Tooltip")
	
    EpeenTooltip.defaultDB = {
        on = true,
		alert = true,
		sound = true,
		cooldown = 1,
		track = true,
    }
	
	Epeen:RegisterDefaults("Tooltip", "profile", opts or EpeenTooltip.defaultDB or {})
	
    Epeen:ToggleModuleActive("Tooltip", self.db.profile.on)
  
	
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
	Epeen.Options.args.settings.args.Tooltip = {
        name = L["Tooltip"],
        desc = L["Toggle showing Epeen information in the tooltip"],
        type = "group",
		order = 106,
        args = { },
	}
	
	Epeen.Options.args.settings.args.Tooltip.args.TooltipHeader = {
		name = bluecolor..L["Tooltip"].."  |c88888888"..Revision,
		icon = "Interface\\Addons\\Epeen\\icon.tga",
		iconHeight = 14,
		iconWidth = 14,
		order = 1,
		type = "header",
    }
	
	Epeen.Options.args.settings.args.Tooltip.args.TooltipHeaderBlank = {
		order = 2,
		type = "header",
	}
	
	Epeen.Options.args.settings.args.Tooltip.args.TooltipTrack = {
        name = L["Track"],
        desc = L["Toggle whether or not to track on Tooltips"],
		order = 50,
        type = "toggle",
        get = function() return self.db.profile.track end,
        set = function() self.db.profile.track = not(self.db.profile.track) end
    }

	Epeen.Options.args.settings.args.Tooltip.args.TooltipAlert = {
        name = L["Alert"],
        desc = L["Toggle whether or not to alert on Tooltips"],
		order = 100,
        type = "toggle",
        get = function() return self.db.profile.alert end,
        set = function() self.db.profile.alert = not(self.db.profile.alert) end
    }
	
	Epeen.Options.args.settings.args.Tooltip.args.TooltipSound = {
        name = L["Play Alert Sound"],
        desc = L["Toggle whether or not to play sound on alert"],
		order = 101,
        type = "toggle",
        get = function() return self.db.profile.sound end,
        set = function() self.db.profile.sound = not(self.db.profile.sound) end
    }
	
	Epeen.Options.args.settings.args.Tooltip.args.TooltipCooldown = {
		name = L["Alert Cooldown"],
		desc = L["Set the Alert Cooldown in minutes"],
		order = 102,
		type = "range",
		min  = 1.0,
		max  = 10.0,
		step = 1,
		set  = function(cooldown) self.db.profile.cooldown = cooldown; end,
		get  = function() return self.db.profile.cooldown end,
    }
				
	Epeen.Options.args.settings.args.Tooltip.args.TooltipToggleBlank = {
       	type = "header",
		order = 195,
	}
	
    Epeen.Options.args.settings.args.Tooltip.args.TooltipToggle = {
    	name = L["Toggle"],
        desc = L["Toggle the Module On and Off"],
		order = 200,
        type = "toggle",
        get = function() return self.db.profile.on end,
        set = function() self.db.profile.on = Epeen:ToggleModuleActive("Alert") end
    }

end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function EpeenTooltip:OnEnable()
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function EpeenTooltip:ResetDB()
	copyTable(self.db.profile, EpeenTooltip.defaultDB)
end

--- ========================================================= ---
--- UPDATE_MOUSEOVER_UNIT
--- ========================================================= ---

function EpeenTooltip:UPDATE_MOUSEOVER_UNIT()
	EpeenTooltip:SetText();
end

--- ========================================================= ---
--- Set Tooltip Text
--- ========================================================= ---

function EpeenTooltip:SetText()
	if (self.db.profile.on == true) then
		
		if( not UnitIsPlayer("mouseover") ) then
   			return;
		end
		local ForeignName, ForeignServer = UnitName("mouseover")
		local guildName, guildRankName, guildRankIndex = GetGuildInfo("mouseover");
		local unitRace, unitRaceuc = UnitRace("mouseover");
		local unitClass, unitClassuc = UnitClass("mouseover");

		if self.db.profile.track then
			self:TriggerEvent("EpeenSeen", UnitFactionGroup("mouseover"), UnitName("mouseover"), ForeignServer, UnitLevel("mouseover"), unitRace, unitClass, guildName, guildRankName)
		end

		local player = UnitName("mouseover");
		local playerlc = string.lower(player);
   
		guildnamelc = nil;
	
		if guildName then
			guildnamelc = string.gsub(string.lower(guildName), "%s", "_");
		end
		
		local AlertNow = false;
		local AlertType = nil;
		local AlertName = nil;
		local AlertReason = nil;
   		
		if (EpeenKoS.db.profile.kosplayer[playerlc]) and EpeenKoS then
			AlertNow = true;
			AlertType = "Player";
			AlertName = player;
			AlertReason = EpeenKoS.db.profile.kosplayer[playerlc].reason;
			
			if AlertReason == "" then
				AlertReason = nil;
			end
			
			if not (AlertReason) then
				GameTooltip:AddLine("Player KoS - "..AlertName, 1.0, 1.0, 1.0);
			else
				GameTooltip:AddLine("Player KoS - "..AlertName.." : "..AlertReason, 1.0, 1.0, 1.0);
			end
		elseif (EpeenFriendly.db.profile.friendlyplayer[playerlc]) and EpeenFriendly then
			AlertNow = false;
			AlertType = "Player";
			AlertName = player;
			AlertReason = EpeenFriendly.db.profile.friendlyplayer[playerlc].reason;
			
			if AlertReason == "" then
				AlertReason = nil;
			end
			
			if not (AlertReason) then
				GameTooltip:AddLine("Player Friendly - "..AlertName, 1.0, 1.0, 1.0);
			else
				GameTooltip:AddLine("Player Friendly - "..AlertName.." : "..AlertReason, 1.0, 1.0, 1.0);
			end
		end
		
		if guildName then
			if (EpeenKoS.db.profile.kosguild[guildnamelc]) and EpeenKoS then
				AlertNow = true;
				AlertType = "Guild";
				AlertName = guildName;
				AlertReason = EpeenKoS.db.profile.kosguild[guildnamelc].reason;
			
				if AlertReason == "" then
					AlertReason = nil;
				end
			
				if not (AlertReason) then
					GameTooltip:AddLine("Guild KoS - "..AlertName, 1.0, 1.0, 1.0);
				else
					GameTooltip:AddLine("Guild KoS - "..AlertName.." : "..AlertReason, 1.0, 1.0, 1.0);
				end
			elseif (EpeenFriendly.db.profile.friendlyguild[guildnamelc]) and EpeenFriendly then
				AlertNow = false;
				AlertType = "Guild";
				AlertName = guildName;
				AlertReason = EpeenFriendly.db.profile.friendlyguild[guildnamelc].reason;
			
				if AlertReason == "" then
					AlertReason = nil;
				end
			
				if not (AlertReason) then
					GameTooltip:AddLine("Guild Friendly - "..AlertName, 1.0, 1.0, 1.0);
				else
					GameTooltip:AddLine("Guild Friendly - "..AlertName.." : "..AlertReason, 1.0, 1.0, 1.0);
				end
			end
		end
		
		if EpeenStats then
			if EpeenStats.db.profile.chars[player] then
				GameTooltip:AddLine("Kills ( "..EpeenStats.db.profile.chars[player].win.." ) - Deaths ( "..EpeenStats.db.profile.chars[player].loss.." )", 1.0, 1.0, 1.0);
			end
		end

   		GameTooltip:Show();
		
		if EpeenTooltipCooldown then
			return;
		end
		
		if (self.db.profile.alert == true) then
			if AlertNow then
				EpeenTooltipCooldown = true;
				
				self:ScheduleEvent("Cooldown", self.Cooldown, (self.db.profile.cooldown * 60), self)
				self:TriggerEvent("EpeenAlert", "Tooltip", AlertType.." KoS", AlertName, AlertReason, self.db.profile.sound)
			end
		end
		
	end
end

--- ========================================================= ---
--- Cooldown
--- ========================================================= ---

function EpeenTooltip:Cooldown()
	EpeenTooltipCooldown = nil;
end